//============================================================================
//
//  XstrmAPI.h: IDT MotionPro header file
//  Version:    2.16.00
//  Copyright (C) 2000-2020 Integrated Design Tools Inc.
//  ALL RIGHTS RESERVED
//
// ---------------------------------------------------------------------------
//  Version   Notes
// ---------------------------------------------------------------------------
//  1.00.00   Created
//  1.01.00   Added binning, pixel gain, noise reduction parameters
//            Added XsCalibrateNoiseReduction API
//  1.02.00   Added Channel balance Parameter
//  1.02.01   Added serial# to XS_ENUM struct and info routine
//            Exposure units changed from micro-seconds to nano-seconds
//            Added XS_IF_GRAY8X image format (8 bit gray from color RGB)
//  1.02.04   Modified the meaning of the parameter nIsFirst in the routines
//            XsMemoryReadData and XsMemoryReadFrame (see documentation).
//  1.03.00   Added XS_CI_MODE (color interpolation algorithm) parameter
//  1.04.00   Added XSI_REVISION info and corresponding field in XS_ENUMITEM
//            Added XSP_PERIOD (camera acquisition period = 1/rate) parameter.
//  1.05.00   Added a parameter to XsCalibrateNoiseReduction API
//            Added XS_IF_GRAY16X image format (16 bit gray from color RGB)
//  1.06.00   Added support for XS-4 (512x512) and XS-5 (1000 fps)
//            Modified the XsSynchGrab prototype
//  1.06.01   Modified sensors models
//  1.07.00   Added noise filtering option
//  1.08.00   Added minimum period parameter [ns]: the driver computes the inverse
//            of the maximum rate allowed by the current configuration (vertical roi
//            and exposure mode).
//            Added internal clck rate info [Hz]
//  1.09.00   Added XSI_CFW_REVISION info (controller Firmware Version)
//  1.10.00   Added XS-6 and XS-7 camera models
//  1.12.00   New architecture
//  1.13.00   Added XsMemoryPreview routine and XSI_EXP_STEPS info
//  1.13.03   Added XSP_IRIG parameter and erased XS_TIC_IRIG
//  1.13.04   Added XSI_INTENSIFIED for intensified cameras
//  1.13.05   Added XsEnableRawMode to read raw bayer data from color cameras
//            Added XS_C_BCKG_NOSAVE for acquiring background images without saving
//  1.15.00   Added XS_LINK_TYPE and XSI_LINK for the link type (USB 2.0 or GigaEthernet)
//            Added nLinkType field to XS_ENUMITEM structure.
//  1.16.00   Added new fields to XS_ENUMITEM structure (link specific).
//  2.00.00   New products names (X models)
//            Added pnAcqFrames parameter to XsMemoryStopGrab.
//            Added support for flash memory. The camera configuration may be
//            read/written to the camera flash.
//            Added XsPreConfigCamera for camera pre configuration (IP address)
//            Added XSP_VIDEO_MODE parameters for PAL/NTSC output video mode selection
//            Added noise calibration options for download/upload of calibration file
//            from/to camera flash memory, and for operation abort.
//  2.00.01   Added video mode capability (info)
//  2.00.02   Added XsWriteUserDataToFlash and XsReadUserDataFromFlash routines for
//            read and write user data buffers to camera flash memory.
//            Added XSPP_NET_AD_IP to set adapter IP address.
//            Added XS_E_NOT_IN_FLASH error code if an item is not in camera
//            flash memory.
//            Added XSI_PLUS information and bPlus field in enumeration struct
//            for Plus camera models (double speed).
//  2.00.03   Added XSP_NET_PERFORM parameter (network performance)
//            Added camera name info (read and write), added giga-ethernet info
//            Added XsSetCameraInfo routine to write info parameters (camera name)
//  2.00.04   Added XsGetCameraStatus routine. it polls the driver to synchronously
//            check the camera status.
//  2.01.00   Added the plus capability (double speed). The operating mode can be
//            activated and deactivated.
//  2.01.02   Added  XSPP_IP_ADD_EX parameter. It allows the configuration of the IP address
//            with the camera MAC address.
//  2.02.00   Added XSI_LINK_FW_VER information. It returns the link (USB/GE) firmware
//            version. Added XSP_GAMMA parameter: the range is 0 to 30. Added XSP_FRAMES
//            and XSP_PRE_TRIG: read the values after an acquisition.
//            Added XSI_CFW2_VERSION information. It returns the second controller firmware
//            version (USB/GE cameras only).
//  2.03.00   Fixed wrong set of exposure when the sync is external and the value is very
//            close to the period T = 1/F. The effect on the images is flashing intensity
//            in the recorded sequence.
//  2.04.00   Added new camera models (Y and M). Added CameraLink Type.
//            Added a parameter to XsGetCameraStatus (busy flag and state).
//            Added camera subnet mask info. Added HG parameters.
//            Added more detailed camera states. Added enumeration filter.
//            Added sync in, sync out, trigger out signals configuration.
//            Added HDMI configuration. 
//  2.04.01   Correct value of XS_EF_ALL mask in the enum filter constants
//  2.04.03   Added XSP_PREV_MODE parameter for full or low res preview (Y cameras)
//            Changed the Sync Out Configuration values XS_SYNCOUT_CFG (Y cameras)
//  2.05.00   Added support for MotionXtra legacy cameras (HG-2000, CR-2000, TX-2000)
//  2.05.04   Added support for Matrox and EPIX PCIe x4 frame-grabbers (M cameras)
//  2.06.00   Added XsAttach to manage simultaneous connections from different computers
//            Added XSP_1ST_FRM_IDX read-only param to read position of first acquired frame.
//            Added XS_C_AUTO_PSC for PSC calculation in current conditions.
//  2.06.01   Added XSP_NOISE_APSC parameter to load current conditions PSC coefficients
//            Added XS_C_AUTO_RESET to reset and delete the current conditions calibration file
//            Modified XSI_NOISE_FILE into XSI_FCAL_FILE_ON (factory calibration file is on)
//            Added XSI_CCAL_FILE_ON (current conditions calibration file is on)
//            Added XSI_FCAL_FILE_FLASH (check if calibration file is stored in flash memory)
//  2.06.03   Added the management of disconnection in status and announcements.
//            "DISCONNE": the camera has been disconnected
//            "RESTORED": the camera has been reconnected
//            "DETACHED": the camera has been detached (attached by another ocmputer)
//  2.06.04   Added N models.
//  2.06.05   Added XsGetAddressList to get frame addresses from N cameras.
//  2.07.00   Added XSPP_CAM_CMD_PORT and XSPP_NET_AD_CMD_PORT pre-config parameters to change
//            Y cameras command port number (default: 1028)
//            N cameras - frame index in XsMemoryReadFrame is the frame size (variable)
//            Added new Look Up Tables.
//            Added XSP_EXT_ parameters for external sync (period and pulse width)
//            Added a parameter to XsMemoryReadTriggerPosition (trigger position time relative to sync period start)
//  2.07.01   Changed prototype of XsPerConfigCamera to port to 64 bit env.
//            Added Bitflow Frame grabber support
//            Added configurable compression ratio for N camera
//            Added max limit to the number of acquired frames in N camera (16380)
//  2.07.02   Added XSP_FRAME_SIZE to compute the size of a frame in camera memory
//  2.07.03   Improved detection of trigger position.
//            Updated max values of frames for N camera
//            Added XSPP_GET_IP_ADDRESS to read IP add and SN mask from HG cameras
//  2.07.05   Added XSI_CAL_PATH to read the camera calibration folder path
//  2.07.08   Added support for Write To Disk (M camera)
//            Added double exposure sync out mode (2 pulses)
//  2.07.10   Added full support for Y6 (12 bit camera)
//            Added 12 to 8 bit Look Up Tables, added bayer pattern info value
//            Added XDR mode for Y4 cameras, added image improvement parameters
//  2.07.12   Added support for IRIG sync (1 PPS).
//            Added flag to detect new design cameras.
//            Added board temperature read for redesigned cameras
//  2.07.13   Added light flag for cameras with limited capabilities.
//            Added the new design flag in the enumeration structure
//  2.07.14   Added XSI_CAL_NAME (calibration file name info)
//            Added XS_HDMI_INDEPENDENT mode to separate camera output from HDMI output
//            Added XsReadToVideo to output frames directly to the HDMI (redesigned cameras only)
//  2.07.16   Added motion trigger parameters (new design cameras only)
//            Added XSI_WRITE_2_DISK info to read if M2D is enabled
//            Added XSI_COMPRESSION to read if camera supports compression
//            Added XSP_DGR_SIZE parameter and XSI_DGR_SIZE info for jumbo packets support.
//            Added bDgrSize item in enumeration structure
//            Added parameter for post-recording operation (XSP_PROP)
//  2.07.18   Added support for auto-exposure (XSI_AUTO_EXPOSURE)
//            Added support for virtual camera (SD card, raw, net disk...)
//            Added info for Post-Rec Operation (XSI_POST_REC_OP)
//            Added parameters to XsGetCameraStatus routine (more info).
//            Added camera default gateway pre-parameter (XSPP_CAM_DFL_GW)
//  2.07.20   Added management of time limited license in camera.
//            Added configurable Sync-Out alignment (XSP_SYNCOUT_ALIGN).
//            Added a parameter to XsReadUserDataFromFlash/XsWriteUserDataToFlash
//            for write/read block of data in/from RAM memory (Y only)
//  2.07.22   Reviewed list of sensor models.
//            Added support for new Y4 and Y5 revisions.
//  2.08.00   Added XSP_MARKER marker option and XSP_CI_THR
//            Added support for M5 with Orion II sensor.
//  2.08.03   Added sub-model information item
//            Added clock speed parameter.
//            Removed Y7 camera model (obsolete)
//            Added support ofr Y5-HD and roi index parameter.
//  2.08.06   Added gaussian filter
//            Added param to XsLoadDriver for activation of USB notifications
//  2.09.00   Added rotation angle (XSP_ROT_ANGLE), flip (XSP_FLIP), Lookup Table (XSP_LUT)
//            Added info to read sensor original pixel depth (XSI_SNS_PIX_DEPTH)
//  2.09.01   Added support for N5-HD, Y3-HD, extra ROI 1504x1128 for Y3C and N3 S1/S2.
//            Added internal/external modes for 1pps sync (RIG/DTS/STP),
//            Added info and support for XDR/EDR.
//            Added XsEnableResize for fast thumbnail live/playback.
//            Added XSP_LUT_CHN_MASK to select RGB channels in LUT.
//  2.09.02   Added XSP_HDMI_OVERLAY parameter (Y cameras only)
//            Added XSP_WBTBL_TEMP parameter for color temperature (Y cameras only)
//            Added hardware configuration info (XSI_HW_BOARDS and XSI_HW_INFO)
//            Added fast preview modes
//            Added XSP_NOISE_DKCOL for the use of masked (dark) columns in noise reduction
//            (removed post filter parameter).
//  2.09.04   Added support for Y6 pipeline + extra ROI of 1600x1200
//  2.09.06   Adapted callback parameters to 64 bit OS.
//            Added support for Y7, added XSP_HD_ZOOM parameters.
//            Added info about EDR support in enum structure.
//            Added TNR info (Temporal Noise Reduction).
//  2.09.07   Added flag for PIV camera. Added threshold for sharpening.
//  2.09.08   Added sync in polarity inversion.
//  2.10.00   Added current calibration info XSI_CUR_CAL, defined new 64 bit integer type
//            Added XsMemoryDownloadRawFrame routine to doanload images in RAW format.
//  2.10.01   Added XSULONG and XSUINT types to support different compilers
//            Added support for JPEG (XSI_JPEG_SUPPORT) and DNR2
//            Added parameter for image zoom (XSP_IMG_ZOOM)
//            Added support for hardware BROC in Y and NR cameras.
//  2.10.02   Added network adapter MTU in the enum structure and in the info
//            Added XSP_HD_ZOOM_2R to disable digital zoom and take ROI image
//            Added 5 more Digital zoom values (from 9X to 16X)
//            Added XsVideoPlayback routine for fast playback on the HDMI output
//            Added support for NR7 and NR5-HD
//  2.10.03   Added XSP_FRAME_SIZE_IMG to calculate image size when donwloaded and saved.
//  2.10.04   Added Y8 and N8 models. Added Miscellanous capabilities info.
//  2.10.05   Added info about NX and NXT in misc capabilities
//            Added info on extra regions of interest (numer and size).
//            Removed unused datagram sizes.
//            Added XSP_SYNC_COUNT parameter that returns the total number of sync from the start of recording.
//  2.11.00   Added configurable Auto-exposure min and max exposure limits
//            Added XS_SENSOR_GAIN, XS_VIDEO_MODE values, added announcement string constants.
//  2.12.00   Added XS_SIC_1PPS (1PPS into sync-in) and XS_SOC_1PPS (1PPS goes on sync-out)
//            Added battery status reporting (XSP_BATTERY_STATUS) and XSI_BATTERYSTS_SUPPORT info.
//            Added XS_LUT_E (pre-defined S-shaped Lookup table) and O-series models
//            Added XSPP_REBOOT_FW to force reboot of camera firmware.
//            Removed hardware info from enum structure and added CFA pattern and build number
//  2.12.02   Added XsOpenRawCamera routine (open a raw file without enumeration)
//  2.12.03   O-series write to disk implementation. Added PIV flag on misc capabilities. Added Os flag.
//            Added GPS module capability and JPEG quality parameter.
//            Added gamma levels and values for multi-gamma curve.
//            Added XsLive for fast live refresh
//  2.12.04   Added support for Os7 speed grades and Y4/Y7/Y8 improved speeds.
//            Added support for 12 bit Os cameras.
//            Save configuration to RAW file is image parameters change.
//  2.12.06   Added HD-SDI modes and USB 3.0 filter for O-series.
//            Added XS_GPSTIMING structure for GPS/IRIG data
//            Added mission readonly parameters (total and executed number of steps)
//            Modified XsReadCameraSettingsArray to read from SSD and from memory
//            Added SSD write progress (read-only) and SSD acquisition index (read/write) parameters.
//            Improved XsGetVersion routine. Removed deprecated XsEnableResize.
//  2.12.07   Added support for Wi-Fi link (O-cameras).
//            Improved trace. Each camera generates its own trace file.
//            Fixed support for YUV format with zoomed images and binning.
//  2.12.08   Added support for PTP synchronization.
//            Added parameter to let the user read IRIG data form a RAW file without loading
//            the full frame (XSP_RAW_RD_TSTAMP).
//            Added PCIe link type and XS_CAP_IS1024 misc flag (the camera supports 1024x1024 res)
//  2.12.09   Added XS_CAP_OS2 and XS_CAP_OS2A capability bits.
//            Added support for CrashCam models.
//  2.12.11   Added XSP_NET_SPEED (HG only) and XSI_TEMP_SUPPORT (temperature support)
//  2.12.12   Os version 2 are actually Os version 3
//  2.12.13   Added XS_ATTR_DEFAULT attribute. Added XSI_CI_THR_SUPPORT info.
//            Removed obsolete cameras. Added new camera models (PCIe, Os9 and CC).
//  2.12.16   Added XS_BROC structure to read array of broc segments from the SSD.
//            Added XS_CAP_PLL flag (Phase Lock Loop support).
//  2.12.17   Added support for Os3 and CC2020.
//            Added external PLL and dynamic pulse modes.
//  2.12.18   Added support to SSD virtual camera.
//  2.12.19   Added lens remote support for O cameras. Added IRIG module capability.
//            Added battery info.
//  2.12.20   Added CCmini 1520 and 3510.
//  2.12.21   Minor changes.
//  2.13.00   Added shock sensor parameters (CC-mini). Added XSP_LATEST_FRM_IDX parameter.
//  2.14.00   Added XSP_LENS_ZOOM (motorized lens). Added paint parameters for XStream camera
//  1.14.01   Added support for CCM-1540 and CCM-3525 models.
//  1.15.00   Added support for Thunderbolt 3 cameras and R cameras.
//            Added deflicker parameters (CC-Mini). Added PTP mode parameter (CC-Mini).
//  1.16.01   Added more Look Up Tables (Gamma, REC-709 and BT-2020). Added 10GE link.
//            Added PTP power up delay. Added XSM 4K, 8K and 4K-X2 models. Added CC-Stick support.
//            Modified XsConfigureWriteToDisk (added support to XSM cameras).
//            Added DDR size and motorized lens information. Lens parameters have been renamed.
//
//============================================================================

#ifndef __XSTREAMVISION_H__
#define __XSTREAMVISION_H__

// Includes
#ifndef XSTREAMAPI
    #ifdef _WIN32
    #define XSTREAMAPI __stdcall
    #else
    #define XSTREAMAPI
    #endif
#endif // XSTREAMAPI

#if defined(_MSC_VER)
typedef unsigned __int64 XSUINT64;
typedef __int64 XSINT64;
typedef unsigned long XSULONG32;
typedef long XSLONG32;
#else
typedef unsigned long long XSUINT64;
typedef long long XSINT64;
typedef unsigned int XSULONG32;
typedef int XSLONG32;
#endif
typedef XSUINT64 *PXSUINT64;
typedef XSINT64 *PXSINT64;
typedef XSULONG32 *PXSULONG32;
typedef XSLONG32 *PXSLONG32;

///////////////////////////////////////////////////////////////////////////////
/// Constant Definitions

// Camera Model
typedef enum
{
    XS_CM_UNKNOWN   = 0,            // Unknown type
    XS_CM_MP_X1     = 1,            // X-Stream XS3
    XS_CM_MP_X4     = 2,            // MotionPro XS4/HS4/X4
    XS_CM_MP_X3     = 3,            // MotionPro XS5/HS3/X3
    XS_CM_MP_X5     = 4,            // MotionPro XS6/X5
    XS_CM_MP_X2     = 5,            // MotionPro HS2
    XS_CM_MP_M3     = 6,            // MotionScope M3
    XS_CM_MP_M4     = 7,            // MotionScope M4
    XS_CM_MP_M5     = 8,            // MotionScope M5
    XS_CM_MP_Y3     = 9,            // MotionPro Y3
    XS_CM_MP_Y4     = 10,           // MotionPro Y4
    XS_CM_MP_Y5     = 11,           // MotionPro Y5
    XS_CM_HG_100K   = 12,           // MotionXtra HG-100K
    XS_CM_HG_LE     = 13,           // MotionXtra HG-LE
    XS_CM_HG_TH     = 14,           // MotionXtra HG-TH
    XS_CM_HG_2K     = 15,           // MotionXtra HG-2000
    XS_CM_CR_2K     = 16,           // MotionXtra CR-2000
    XS_CM_TX_2K     = 17,           // MotionXtra TX-2000
    XS_CM_MP_N3     = 18,           // MotionXtra N3/NR3/NX3
    XS_CM_MP_N4     = 19,           // MotionXtra N4/NR4/NX4
    XS_CM_MP_N5     = 20,           // MotionXtra N5/NR5/NX5
    XS_CM_MP_Y6     = 21,           // MotionPro Y6
    XS_CM_MP_Y7     = 22,           // MotionPro Y7
    XS_CM_MP_N7     = 23,           // MotionXtra NR7/NX7
    XS_CM_MP_Y8     = 24,           // MotionPro Y8
    XS_CM_MP_N8     = 25,           // MotionXtra NX8
    XS_CM_MP_Y10    = 26,           // MotionXtra Y10
    XS_CM_MP_O9     = 27,           // MotionXtra Os9
    XS_CM_MP_O4     = 28,           // MotionXtra Os4
    XS_CM_MP_O5     = 29,           // MotionXtra Os5
    XS_CM_MP_O10    = 30,           // MotionXtra Os10
    XS_CM_MP_O7     = 31,           // MotionXtra Os7
    XS_CM_MP_O8     = 32,           // MotionXtra Os8
    XS_CM_CC_1060   = 33,           // CrashCam 1060
    XS_CM_CC_1520   = 34,           // CrashCam 1520
    XS_CM_CC_1540   = 35,           // CrashCam 1540
    XS_CM_CC_4010   = 36,           // CrashCam 4010
    XS_CM_CC_M1510  = 37,           // CrashCam Mini (1510)
    XS_CM_CC_2020   = 38,           // CrashCam 2020
    XS_CM_CC_M5K05  = 39,           // CrashCam Mini 5K05
    XS_CM_MP_O3     = 40,           // MotionXtra Os3
    XS_CM_CC_M1520  = 41,           // CrashCam Mini 1520
    XS_CM_CC_M3510  = 42,           // CrashCam Mini 3510
    XS_CM_MINI_HD   = 43,           // Mini HD (POV)
    XS_CM_CC_M1540  = 44,           // CrashCam Mini 1540
    XS_CM_CC_M3525  = 45,           // CrashCam Mini 3525
    XS_CM_PCIE_X7   = 46,           // PCIe X-Stream 720p
    XS_CM_PCIE_X14  = 47,           // PCIe X-Stream 1440p
    XS_CM_XSM_1540  = 48,           // TB X-Stream Mini 1540
    XS_CM_XSM_3520  = 49,           // TB X-Stream Mini 3520
    XS_CM_XSM_STICK = 50,           // TB X-Stream Mini Stick
    XS_CM_XSM_5K    = 51,           // TB X-Stream Mini 5K
    XS_CM_XSM_4KV   = 52,           // TB X-Stream Mini 4K-Veloce
    XS_CM_XSM_HD    = 53,           // TB X-Stream Mini HD
    XS_CM_R_2K      = 55,           // R-series 2K
    XS_CM_CC_STICK  = 56,           // CrashCam Stick
} XS_CAM_MODEL;

// backward compatibility items
#define XS_CM_XS3_USB2            XS_CM_MP_X1
#define XS_CM_XS4_USB2            XS_CM_MP_X4
#define XS_CM_XS5_USB2            XS_CM_MP_X3
#define XS_CM_XS6_USB2            XS_CM_MP_X5
#define XS_CM_XS7_USB2            XS_CM_MP_X2

// camera name max lenght
#define L_CAMNAME                 128

// max number of frames that can be acquired by old design N3 and N4 cameras
#define MAX_N_ACQ_FRAMES          16352

// Enumeration filter
typedef enum
{
    XS_EF_USB_X = 0x00000001,     // MotionPro XS/HS/X (USB 2.0)
    XS_EF_GE_X  = 0x00000002,     // MotionPro X (Giga-Ethernet)
    XS_EF_HG    = 0x00000004,     // MotionXtra HG (Giga-Ethernet)
    XS_EF_CL    = 0x00000008,     // MotionScope M (Camera-Link)
    XS_EF_USB_Y = 0x00000010,     // MotionPro Y (USB 2.0)
    XS_EF_GE_Y  = 0x00000020,     // MotionPro Y (Giga-Ethernet)
    XS_EF_LG_RL = 0x00000040,     // Legacy MotionXtra
    XS_EF_GE_N  = 0x00000080,     // MotionXtra N/NR/NX/O (Giga-Ethernet)
    XS_EF_GE_NO = 0x00000080,     // see above
    XS_EF_PCI_X = 0x00000100,     // X-Stream PCIe/Thunderbolt
    XS_EF_VCAM  = 0x00010000,     // Virtual camera (RAW sequence)
    XS_EF_VSSD  = 0x00020000,     // Virtual camera (removable SSD, SD card)
    //
    XS_EF_ALL   = 0x000001FF,     // mask
} XS_ENUM_FLT;

// Link type
typedef enum
{
    XS_LT_USB20   = 0,            // USB 2.0
    XS_LT_GIGAETH = 1,            // Gigabit Ethernet
    XS_LT_CAMLINK = 2,            // Camera Link
    XS_LT_SDCARD  = 3,            // SD card/SSD (virtual camera)
    XS_LT_RAWFILE = 4,            // RAW file (virtual camera)
    XS_LT_WIFI    = 5,            // WiFi
    XS_LT_PCIE    = 6,            // PCIe
    XS_LT_TB      = 7,            // Thunderbolt
    XS_LT_10GIGAETH = 8,          // 10 Gigabit Ethernet
} XS_LINK_TYPE;

// Frame Grabber Type (Camera Link only)
typedef enum
{
    XS_FG_COR_X64CL   = 0,        // Dalsa-Coreco X64 Xcelera-CL PX4
    XS_FG_NI_PCIE1429 = 1,        // National Instruments PCIe-1429
    XS_FG_MATROX_HS   = 2,        // Matrox Helios/Solios
    XS_FG_EPIXCI_E4   = 3,        // Epix PIXCI E4
    XS_FG_BF_KARBON   = 4,        // Bitflow Karbon CL
} XS_FG_TYPE;

// Sensor type (monochrome/color)
typedef enum
{
    XS_ST_MONOCHROME = 0,         // monochrome sensor (gray scale)
    XS_ST_COLOR      = 1,         // color sensor (bayer pattern)
} XS_SNS_TYPE;

// CFA Bayer Pattern
typedef enum
{
    XS_CFAP_GRBG = 0,             // Green on Red
    XS_CFAP_BGGR = 1,             // Blue
    XS_CFAP_RGGB = 2,             // Red
    XS_CFAP_GBRG = 3,             // Green on Blue
} XS_CFA_PATTERN;

// Sensor Model
typedef enum
{
    XS_SM_UNKNOWN       = 0,      // Unknown model
    XS_SM_MV13          = 1,      // MV-13 (XS3)
    XS_SM_MV02          = 2,      // MV-02 (HS4/X4)
    XS_SM_RL_LEGACY     = 3,      // Redlake legacy (HG-2000, CR, TX)
    XS_SM_MAKO          = 4,      // MAKO (HG100K, LE, TH)
    XS_SM_SIRIUS        = 5,      // Sirius (X3, M3, Y3)
    XS_SM_ORION         = 6,      // Orion (X5, M5, Y5, N5)
    XS_SM_ORION_II      = 7,      // Orion II (Y5, N5, NR5, NX5)
    XS_SM_NORTH_STAR    = 8,      // North Star (Y4)
    XS_SM_NORTH_STAR_II = 9,      // North Star II (Y4, N4, NR4, NX4, Y3S1, Y3S2, N3S3)
    XS_SM_NOZOMI        = 10,     // Nozomi (Y6)
    XS_SM_PEGASUS       = 11,     // Pegasus (Y10/O10)
    XS_SM_SIRIUS_II     = 12,     // Sirius II (O7)
    XS_SM_GEMINI        = 13,     // Gemini (O9)
    XS_SM_LEO           = 14,     // Leo (CCM-5K/XSM-5K)
    XS_SM_STK           = 15,     // Stick sensor
    XS_SM_RCHD          = 16,     // R-HD sensor
    XS_SM_4KVEL         = 17,     // 4K veloce sensor
} XS_SNS_MODEL;

// Camera Revisions
typedef enum
{
    XS_REV_A = 0,
    XS_REV_B = 1,
    XS_REV_C = 2,
    XS_REV_D = 3,
    XS_REV_E = 4,
    XS_REV_F = 5,
    XS_REV_G = 6,
    XS_REV_H = 7,
    XS_REV_I = 8,
    XS_REV_J = 9,
} XS_REVISION;

// Miscellaneous capabilities
typedef enum
{
    XS_CAP_NR     = 0x00000001,   // the camera is NR
    XS_CAP_NX     = 0x00000002,   // the camera is NX
    XS_CAP_NXT    = 0x00000004,   // the camera is NXTra
    XS_CAP_NXA    = 0x00000008,   // the camera is NXAir
    XS_CAP_DNR2   = 0x00000010,   // the camera supports DNR2
    XS_CAP_HWBROC = 0x00000020,   // the camera supports HW BROC
    XS_CAP_JPEG   = 0x00000040,   // the camera supports JPEG
    XS_CAP_1PPS   = 0x00000080,   // the camera supports 1PPS sync mode (in and out)
    XS_CAP_BATSTS = 0x00000100,   // the camera supports battery status read
    XS_CAP_FBCAM  = 0x00000200,   // the camera is FB cam (Forward Bay Camera)
    XS_CAP_PIV    = 0x00000400,   // the camera is PIV ready
    XS_CAP_OS     = 0x00000800,   // the camera is Os (sealed O-series)
    XS_CAP_GPSMOD = 0x00001000,   // the camera has internal GPS module
    XS_CAP_INX    = 0x00002000,   // the camera is iNdustrial camera
    XS_CAP_JPLROC = 0x00004000,   // the camera is JPL ROC
    XS_CAP_PTP    = 0x00008000,   // the camera supports PTP
    XS_CAP_IS1024 = 0x00010000,   // the camera is Y4,N4,NR4 or NX4 that supports 1024x1024 resolution (not only 1016x1016)
    XS_CAP_OS3    = 0x00020000,   // the camera is Os3 (Os-series version 3)
    XS_CAP_OSA    = 0x00040000,   // the camera is OsA (Airborne)
    XS_CAP_PLL    = 0x00080000,   // the camera supports Phase Lock Loop (PLL)
    XS_CAP_IRIGMD = 0x00100000,   // the camera has internal IRIG module
    XS_CAP_SDI_FW = 0x00200000,   // the camera has a special firmware for SDI
    XS_CAP_OSTRM  = 0x00400000,   // the camera is OStreaming
} XS_MISC_CAPS;

#define CAP_IS_NR(c)     ((c)&XS_CAP_NR)
#define CAP_IS_NX(c)     ((c)&XS_CAP_NX)
#define CAP_IS_NTRA(c)   ((c)&XS_CAP_NXT)
#define CAP_IS_NAIR(c)   ((c)&XS_CAP_NXA)
#define CAP_IS_FBCAM(c)  ((c)&XS_CAP_FBCAM)
#define CAP_IS_OS(c)     ((c)&XS_CAP_OS)
#define CAP_IS_INX(c)    ((c)&XS_CAP_INX)
#define CAP_IS_ROC(c)    ((c)&XS_CAP_JPLROC)
#define CAP_IS_OS3(c)    ((c)&XS_CAP_OS3)
#define CAP_IS_OSA(c)    ((c)&XS_CAP_OSA)

// pre config parameters
typedef enum
{
    XSPP_IP_ADDRESS      = 0,     // camera IP address
    XSPP_NET_AD_IP       = 1,     // network adapter IP address (all cameras)
    XSPP_IP_ADD_EX       = 2,     // camera IP address when the camera is not enumerated
    XSPP_CAM_CMD_PORT    = 3,     // camera command UDP port (listening)
    XSPP_NET_AD_CMD_PORT = 4,     // local application command UDP port (sending)
    XSPP_GET_IP_ADDRESS  = 5,     // get ip address and network mask (HG only)
    XSPP_DB_FOLDER       = 6,     // DB folder, for RAW virtual cameras detection
    XSPP_CAM_DFL_GW      = 7,     // camera Default Gateway
    XSPP_DISABLE_1024    = 8,     // disable 1024x1024 resolution on new Y4,N4,NR4,NX4 cameras
    XSPP_REBOOT_FW       = 9,     // reboot camera firmware
    XSPP_PCIX_DMASIZE    = 10,    // configure size of DMA buffer of PCIe camera
} XS_PRE_PARAM;

// camera status
typedef enum
{
    XSST_UNKNOWN     = 0,         // unknown status
    XSST_IDLE        = 1,         // the camera is idle (no operation)
    XSST_LIVE        = 2,         // the camera is in live mode (HG-only)
    XSST_REC_PRETRG  = 3,         // the camera is recording before trigger
    XSST_REC_POSTRG  = 4,         // the camera is recording after trigger
    XSST_REC_DONE    = 5,         // the camera has recorded images in memory
    XSST_DOWNLOAD    = 6,         // the camera is downloading images (HG-only)
    XSST_DISCONNECT  = 7,         // the camera is disconnected
    XSST_DWL_SD      = 8,         // the camera is downloading memory into SD card
    XSST_DWLUPL_SSD  = 8,         // the camera is downloading/uploading from/to SSD
    XSST_VPB_ON_COFF = 9,         // Video Playback is on but HDMI cable is disconnected
    XSST_VPB_ON_CON  = 10,        // Video Playback is on and HDMI cable is on
    XSST_PLAYBACK    = 11,        // the camera is in automatic playback mode (Mini HD)
} XS_STATUS;

// Exposure mode
typedef enum
{
    XS_EM_SINGLE_EXP = 0,         // single exposure
    XS_EM_DOUBLE_EXP = 1,         // double exposure
    XS_EM_EDR        = 2,         // Extended Dynamic Range (Y4,N4 cameras)
    XS_EM_XDR        = 3,         // eXtended Dynamic Range (Y/N cameras, obsolete)
} XS_EXP_MODE;

// Record mode
typedef enum
{
    XS_RM_NORMAL   = 0,           // Normal mode (fills the memory segment and stop)
    XS_RM_CIRCULAR = 1,           // Circular mode (fills the memory segment and restart)
    XS_RM_BROC     = 2,           // Burst Record On Command
    XS_RM_ROC      = 3,           // Record On Command (HG-only)
    XS_RM_READY    = 4,           // Ready (HG-only)
} XS_REC_MODE;

// Sync In configuration (on 2 BNC cameras it configures the trigger also)
typedef enum
{
    // internal
    XS_SIC_INTERNAL     = 0,      // acquisition is synchronized by internal clock
    // external
    XS_SIC_EXT_EDGE_HI  = 1,      // acquisition is synchronized by rising sync in edge
    XS_SIC_EXT_EDGE_LO  = 2,      // acquisition is synchronized by falling sync in edge
    XS_SIC_EXT_PULSE_HI = 3,      // acquisition is synchronized by pulse size (high level)
    XS_SIC_EXT_PULSE_LO = 4,      // acquisition is synchronized by pulse size (low level)
    // IRIG/GPS/DTS/STP
    XS_SIC_IRIG_DTS_EXT = 5,      // acquisition is synced by IRIG or GPS signal (model dependent)
    XS_SIC_IRIG_DTS_INT = 6,      // acquisition is synced by IRIG or GPS signal (model dependent)
    XS_SIC_1PPS         = 7,      // acquisition is synced by 1PPS signal entering from the sync in connector
    XS_SIC_PTP          = 8,      // acquisition is synced by Precision Time Protocol
    // new external modes
    XS_SIC_EPLL_EDGE_HI  = 20,    // external edge high - phase lock loop
    XS_SIC_EPLL_EDGE_LO  = 21,    // external edge low - phase lock loop
    XS_SIC_EDYN_PULSE_HI = 22,    // external pulse high - dynammic (pulse width changes)
    XS_SIC_EDYN_PULSE_LO = 23,    // external pulse low - dynamic (pulse width changes)
} XS_SYNCIN_CFG;

// Sync Out configuration
typedef enum
{
    XS_SOC_DFL        = 0,        // default cfg: sync out follows rate and exposure
    XS_SOC_DFL_INV    = 1,        // the default sync out signal is inverted
    XS_SOC_CFGWID     = 2,        // the sync out width is configurable
    XS_SOC_CFGWID_INV = 3,        // the sync out width is configurable and the signal is inverted
    XS_SOC_DISABLED   = 4,        // the sync out signal is disabled
    XS_SOC_DBLEXP     = 5,        // sync out reflects the double exposure
    XS_SOC_1PPS       = 6,        // sync out signal is a 1 PPS signal

} XS_SYNCOUT_CFG;

// Sync Out alignment
typedef enum
{
    XS_SOA_EXP        = 0,        // default cfg: sync out is aligned to exposure
    XS_SOA_SYNC_IN    = 1,        // sync out is aligned to sync-in

} XS_SYNCOUT_ALIGN;

// Trigger In Configuration (external trigger-in signal on 3 BNC camera models)
typedef enum
{
    XS_TIC_EDGE_HI  = 0,          // Trigger starts on edge high (rising)
    XS_TIC_EDGE_LO  = 1,          // Trigger starts on egge low (falling)
    XS_TIC_SWC      = 2,          // Trigger starts on switch closure
    XS_TIC_GATE_HI  = 3,          // Trigger acts as a gate when the level is high
    XS_TIC_GATE_LO  = 4,          // Trigger acts as a gate when the level is low
    XS_TIC_DISABLED = 5,          // Trigger is disabled
} XS_TRIGIN_CFG;

// Motion Trigger Mode
typedef enum
{
    XS_MT_DISABLED = 0,           // Motion Trigger disabled
    XS_MT_AVG_CHG  = 1,           // Trigger if average luminanace changes
    XS_MT_AVG_INCR = 2,           // Trigger if average luminanace increases
    XS_MT_AVG_DECR = 3,           // Trigger if average luminanace decreases
    XS_MT_MOTION   = 4,           // Trigger if simple motion is detected (no change of intensity)
} XS_MTRIG_CFG;

// Image format
typedef enum
{
    XS_IF_GRAY8   = 0,            // Gray 8: data is bytes
    XS_IF_BAYER8  = 0,            // Bayer pattern 8 bit: data is bytes
    XS_IF_GRAY16  = 1,            // Gray 16: data is 16 bit word, LSB aligned
    XS_IF_BAYER16 = 1,            // Bayer pattern 16: data is 16 bit word, LSB aligned
    XS_IF_BGR24   = 2,            // Windows 24 bit BGR
    XS_IF_BGRA32  = 3,            // Windows 32 bit BGRA (A is not used)
    XS_IF_ARGB32  = 4,            // MAC 32 bit ARGB (A is not used)
    XS_IF_GRAY8X  = 5,            // Gray 8 converted from RGB
    XS_IF_GRAY16X = 6,            // Gray 16 cnverted from RGB
    XS_IF_BGR48   = 7,            // Windows 48 bit BGR: each component is 16 bit word, LSB aligned
} XS_IMG_FMT;

// color interpolation mode
typedef enum
{
    XS_CIM_BILINEAR    = 0,       // bilinear demosaic
    XS_CIM_ADVANCED    = 1,       // advanced high performance demosaic
    XS_CIM_ADVANCED2   = 2,       // more advanced demosaic
} XS_CI_MODE;

// sensor digital gain
typedef enum
{
    XS_SG_1_00 = 0,               // no gain (gain = 1.00)
    XS_SG_1_41 = 1,               // sqrt(2) (gain = 1.44)
    XS_SG_2_00 = 2,               // 2 (gain = 2.00)
    XS_SG_2_82 = 3,               // 2*sqrt(2) (gain = 2.82)
} XS_SENSOR_GAIN;

// Pixel Gain
typedef enum
{
    XS_PG_1X      = 0,            // Bits 2-9 (upper)
    XS_PG_2X      = 1,            // Bits 1-8 (middle)
    XS_PG_4X      = 2,            // Bits 0-7 (lower)
} XS_PIXEL_GAIN;

// Lookup Table
typedef enum
{
    XS_LUT_OFF    = 0,            // Disabled LUT
    XS_LUT_USER   = 1,            // User-defined LUT
    XS_LUT_A      = 2,            // Pre-defined LUT: A
    XS_LUT_B      = 3,            // Pre-defined LUT: B
    XS_LUT_C      = 4,            // Pre-defined LUT: C
    XS_LUT_D      = 5,            // Pre-defined LUT: D
    XS_LUT_E      = 6,            // Pre-defined LUT: E
    XS_LUT_G14    = 7,            // Pre-defined LUT: Gamma 1.4
    XS_LUT_G18    = 8,            // Pre-defined LUT: Gamma 1.8
    XS_LUT_G20    = 9,            // Pre-defined LUT: Gamma 2.0
    XS_LUT_REC709 = 10,           // Pre-defined LUT: REC 709
    XS_LUT_BT2020 = 11,           // Pre-defined LUT: BT 2020
} XS_LUT;

// Lut channels mask
typedef enum
{
    XS_LUTMSK_OFF = 0x00000000,   // disabled channels
    XS_LUTMSK_R   = 0x00000001,   // enable red channel
    XS_LUTMSK_G   = 0x00000002,   // enable green channel
    XS_LUTMSK_B   = 0x00000004,   // enable blue channel
    XS_LUTMSK_ALL = 0x00000007,   // enable all channels
} XS_LUT_MASK;

// Binning
typedef enum
{
    XS_BIN_1X1 = 0,               // Binning 1x1
    XS_BIN_2X2 = 1,               // Binning 2x2
    XS_BIN_3X3 = 2,               // Binning 3x3
    XS_BIN_4X4 = 3                // Binning 4x4
} XS_BINNING;

// HDMI mode (Y cameras)
typedef enum
{
    XS_HDMI_OFF         = 0,      // HDMI is off
    XS_HDMI_ON          = 1,      // HDMI is on, recorded and live images are sent to PC and HDMI output
    XS_HDMI_TRANSFER    = 2,      // HDMI is on, recorded and live images are sent to HDMI output (PC output is disabled)
    XS_HDMI_INDEPENDENT = 3,      // HDMI is on and independent from PC (new designed cameras only)
} XS_HDMI_MODE;

// Video Output mode (X, HG-legacy, Y and O cameras)
typedef enum
{
    XS_VM_X_PAL        = 0,       // X/HG camera: PAL output
    XS_VM_Y_720P_60HZ  = 0,       // Y camera: 720p 60 Hz
    XS_VM_O_1080P_60HZ = 0,       // O camera: 1080p 60 Hz
    XS_VM_X_NTSC       = 1,       // X/HG camera: NTSC output
    XS_VM_Y_1080_60HZ  = 1,       // Y camera: 1080p 60 Hz
    XS_VM_O_1080P_50HZ = 1,       // O camera: 1080p 50 Hz
    XS_VM_Y_1080_25HZ  = 2,       // Y camera: 1080p 25 Hz
    XS_VM_O_1080P_30HZ = 2,       // O camera: 1080p 30 Hz
    XS_VM_Y_1080_24HZ  = 3,       // Y camera: 1080p 24 Hz
    XS_VM_O_1080P_25HZ = 3,       // O camera: 1080p 25 Hz
    XS_VM_Y_1080_30HZ  = 4,       // Y camera: 1080p 30 Hz
    XS_VM_O_1080P_24HZ = 4,       // O camera: 1080p 24 Hz
    XS_VM_O_720P_60HZ  = 5,       // O camera: 720p 60 Hz
    XS_VM_O_720P_50HZ  = 6,       // O camera: 720p 50 Hz
    XS_VM_O_720P_30HZ  = 7,       // O camera: 720p 30 Hz
    XS_VM_O_720P_25HZ  = 8,       // O camera: 720p 25 Hz
    XS_VM_O_720P_24HZ  = 9,       // O camera: 720p 24 Hz
} XS_VIDEO_MODE;

// Video playback modes (Y cameras)
typedef enum
{
    XS_VPB_OFF = 0,               // video playback off
    XS_VPB_FWD = 1,               // video playback forward
    XS_VPB_REW = 2,               // video playback rewind
} XS_VIDEO_PB;

// Preview Modes (Y and N/NR/NX cameras)
typedef enum
{
    XS_PM_FULL_RES = 0,           // full resolution preview
    XS_PM_LOW_RES  = 1,           // low resolution preview (decimated)
} XS_PREV_MODE;

// Live commands
typedef enum
{
    XS_LIVE_STOP  = 0,            // stop live
    XS_LIVE_START = 1,            // start live
} XS_LIVE;

// Callback Flag
typedef enum
{
    XS_CF_DONE = 1,               // the callback is called if the operation is successfull
    XS_CF_FAIL = 2,               // the callback is called if the operation is NOT successfull
    XS_CF_CBONLY = 4,             // install callback only and do not start recording
} XS_CALLBACK_FLAGS;

// Calibration op codes
typedef enum
{
    XS_C_BKG_ALL       = 0,       // Calibrate background in all conditions (don't overwrite factory file)
    XS_C_FILE_RELOAD   = 1,       // Reload factory calibration file
    XS_C_FILE_DOWNLOAD = 2,       // Download calibration file from the camera to the disk
    XS_C_CURRENT_BKG   = 3,       // Calibrate background in current conditions
    XS_C_CURRENT_PSC   = 4,       // Calibrate pixel sensitivity correction (PSC) in current conditions
    XS_C_CURRENT_RESET = 5,       // Reset current conditions coefficients and delete file
    XS_C_MEMORY        = 6,       // Memory calibration (CC-Mini)
    XS_C_ABORT         = 100,     // Abort any of the above procedures
} XS_CALIB_OPCODE;

// GE datagram sizes
typedef enum
{
    XS_DGR_1448  = 0,             // default: regular datagram size
    XS_DGR_2888  = 1,
    XS_DGR_4328  = 2,
    XS_DGR_5768  = 3,
    XS_DGR_7208  = 4,
    XS_DGR_8648  = 5,             // jumbo packet max size
} XS_DGR_SIZE;

// Post record operation (operation performed after a recording)
typedef enum
{
    XS_PR_NOTHING    = 0,         // do nothing
    XS_PR_DWL_SD     = 1,         // download to SD card in IRIG-Flash module
    XS_PR_SSD_STREAM = 2,         // download to SSD in stream mode, synchronous (O-series)
    XS_PR_RESERVED1  = 3,         // reserved
    XS_PR_RESERVED2  = 4,         // reserved
    XS_PR_SSD_BACKUP = 5          // download to SSD in backup mode, asynchronous (O-series)

} XS_PR_OP;

// Marker Configuration
typedef enum
{
    XS_MRK_OFF    = 0,            // disabled
    XS_MRK_SYNCIN = 1,            // enabled on sync in
    XS_MRK_TRIGIN = 2,            // enabled on trigger in
} XS_MARKER_CFG;

// Clock speed (Y5 camera only)
typedef enum
{
    XS_CLS_LOWER   = 0,           // Lower speed - better image quality
    XS_CLS_MIDDLE  = 1,           // Middle speed - average image quality
    XS_CLS_HIGHER  = 2,           // Higher speed - lower image quality
} XS_CLOCK_SPEED;

// pre-defined ROI for HD cameras (different meanings)
typedef enum
{                     //   Y3-HD   * Y5/N5/Os5 * Y3C/N3S1-2 *    Y6     * Os10/O10
    XSHD_RES_00 = 0,  // 1920x1080      -           -             -           -
    XSHD_RES_01 = 1,  //  1280x720      -           -             -           -
    XSHD_RES_02 = 2,  //     -       2560x1920   1504x1128    1600x1200   2560x1600
    XSHD_RES_03 = 3,  //     -       2560x1440       -        1920x1080   2560x1440
    XSHD_RES_04 = 4,  //     -       2560x1080       -            -       1920x1080
    XSHD_RES_05 = 5,  //     -           -           -            -        1280x720
} XS_HD_ROI;

// digital zoom factors for HD cameras (Dynamic Magnification)
typedef enum
{
    XSHD_Z_100 = 0,               // 1X
    XSHD_Z_125 = 1,               // 1.25X
    XSHD_Z_150 = 2,               // 1.50X
    XSHD_Z_200 = 3,               // 2X
    XSHD_Z_300 = 4,               // 3X
    XSHD_Z_400 = 5,               // 4X
    XSHD_Z_500 = 6,               // 5X
    XSHD_Z_600 = 7,               // 6X
    XSHD_Z_700 = 8,               // 7X
    XSHD_Z_800 = 9,               // 8X
    XSHD_Z_900 = 10,              // 9X
    XSHD_Z_1000 = 11,             // 10X
    XSHD_Z_1200 = 12,             // 12X
    XSHD_Z_1400 = 13,             // 14X
    XSHD_Z_1600 = 14,             // 16X
} XS_HD_ZOOM;

// rotation angle
typedef enum
{
    XS_ROT_0   = 0,               // no rotation
    XS_ROT_90  = 1,               // 90 degrees clockwise
    XS_ROT_180 = 2,               // 180 degrees
    XS_ROT_270 = 3,               // 90 degrees counter-clockwise
} XS_ROT_ANGLE;

// image flip
typedef enum
{
    XS_FLIP_NONE = 0,             // no flip
    XS_FLIP_HORZ = 1,             // horizontal flip
    XS_FLIP_VERT = 2,             // vertical flip
    XS_FLIP_BOTH = 3,             // flip on both directions
} XS_FLIP;

// JPEG Compression
typedef enum
{
    XS_JPEG_OFF = 0,              // disable JPEG
    XS_JPEG_CVT = 1,              // enable uncompressed JPEG (JPEG is downloaded from camera and decompressed in the driver)
    XS_JPEG_RAW = 2,              // enable compressed JPEG (JPEG is downloaded from camera and read from SDK)
} XS_JPEG;

// battery status (NX/Os/CC cameras)
typedef enum
{
    XS_BAT_LEVEL_MASK          = 0x000000ff,  // battery level mask 0..100
    XS_BAT_STATE_MASK          = 0x00000f00,  // battery state mask
    XS_BAT_STATE_DISCHARGING   = 0x00000000,  // power supply is not connected, running on battery
    XS_BAT_STATE_CHARGING      = 0x00000100,  // power supply is connected, battery is charging
    XS_BAT_STATE_FULLY_CHARGED = 0x00000200,  // power supply is connected, battery is fully charged
    XS_BAT_DISABLED            = 0x00008000,  // the battery is connected but it's disabled
    XS_BAT_UNKNOWN             = 0xffffffff,  // no battery installed or problem communicating with battery
} XS_BATTERY;

// motorized lens info
typedef enum
{
    XS_LENS_TYPE       = 0x0000000F,  // lens type (4 bit) 0:no lens 1:Canon 2:MFT ...
    XS_LENS_ZOOM       = 0x00000010,  // lens has configurable zoom (0:Prime)
    XS_LENS_MOTOR_ZOOM = 0x00000020,  // lens has motorized zoom  
} XS_LENS_INFO;

// motorized lens commands
typedef enum
{
	XS_LCMD_POWEROFF = 0, // lens power off
	XS_LCMD_RESET    = 1, // lens reset
} XS_LENS_CMD;

// Announcement strings (Y,N,NX,NR cameras)
#define XS_ANN_STR_DISCONN  "DISCONNE"  // the camera is disconnected
#define XS_ANN_STR_RESTORE  "RESTORED"  // the camera is conencted after disconnection
#define XS_ANN_STR_REBOOT   "REBOOTED"  // the camera is connected after disconnection and reboot
#define XS_ANN_STR_DETACH   "DETACHED"  // the camera is detached (another computer has attched it)
#define XS_ANN_STR_IDLE     "IDLEMODE"  // the camera is in idel mode (not recording)
#define XS_ANN_STR_PRETRG   "PRE_TRIG"  // the camera is recording and waiting for trigger
#define XS_ANN_STR_POSTTRG  "POSTTRIG"  // the camera has received the trigger and it's recording post-trigger frames
#define XS_ANN_STR_DISKSAVE "DISKSAVE"  // the camera is saving the images to the SSD
#define XS_ANN_STR_PLAYBACK "PLAYBACK"  // the camera is in automatic playback mode

// Return Codes
typedef enum
{
    XS_SUCCESS             = 0,         // OK!!!
    //
    XS_E_GENERIC_ERROR     = 1,         // Generic error
    //
    XS_E_NOT_SUPPORTED     = 2,         // Function is not supported for this device
    XS_E_INVALID_VALUE     = 3,         // Invalid parameter value
    XS_E_INVALID_CFG       = 4,         // Invalid XS_SETTINGS struct
    XS_E_INVALID_HANDLE    = 5,         // Invalid XS_HANDLE
    XS_E_INVALID_CAMERA_ID = 6,         // Invalid camera ID used in XsOpenCamera
    XS_E_INVALID_ARGUMENTS = 7,         // Invalid function arguments
    XS_E_READONLY          = 8,         // the parameter is read-only
    XS_E_CAM_ALREADY_OPEN  = 9,         // the camera is already open
    XS_E_HARDWARE_FAULT    = 10,        // Hardware fault (severe)
    XS_E_BUSY              = 11,        // The camera is busy
    XS_E_QUEUE_FULL        = 12,        // Cannot queue more items, queue is full
    XS_E_BUFFER_TOO_SMALL  = 13,        // The buffer is too small for an acquisition
    XS_E_TIMEOUT           = 14,        // Operation time-out
    XS_E_NOT_RECORDING     = 15,        // the camera is not in record mode
    XS_E_MALLOC            = 16,        // error allocating memory
    XS_E_ABORTED           = 17,        // error because a procedure has been aborted
    XS_E_NOT_IN_FLASH      = 18,        // the requested information is not stored in flash memory
    XS_E_EXP_LICENSE       = 19,        // the time limited license is expired
    XS_E_W2D_OVERRUN       = 20,        // write to disk overrun (O-series/XS-Mini)
    XS_E_DMA_OVERRUN       = 21,        // DMA overrun error (PCIe/XS-Mini)
    XS_E_FILE_OPEN         = 22,        // Unable to open file for streaming to local disk
    XS_E_DISK_FULL         = 23,        // Local disk is full (unable to keep streaming)
} XS_ERROR;

// Camera Parameters Index
typedef enum
{
    XSP_GAIN         = 0,               // Camera sensor gain (see XS_SENSOR_GAIN)
    XSP_EXPOSURE     = 1,               // Exposure in nanoseconds
    XSP_IMG_FORMAT   = 2,               // Image format (See XS_IMG_FMT)
    XSP_PIX_DEPTH    = 3,               // Pixel depth: 8/9/10/11/12 (mono) 24/27/30/33/36 (colour)
    XSP_PIX_GAIN     = 4,               // Pixel gain (see XS_PIXEL_GAIN)
    XSP_SYNCIN_CFG   = 5,               // sync in configuration (see XS_SYNCIN_CFG)
    XSP_REC_MODE     = 6,               // recording mode (see XS_REC_MODE)
    XSP_EXP_MODE     = 7,               // Exposure Mode (see XS_EXP_MODE)
    XSP_BINNING      = 8,               // Binning (See XS_BINNING)
    XSP_CI_MODE      = 9,               // Color interpolation mode (see XS_CI_MODE)
    XSP_TRIGIN_CFG   = 10,              // trigger in configuration (see XS_TRIGIN_CFG)
    XSP_MAX_WIDTH    = 11,              // Maximum image width (binning dependent)
    XSP_MAX_HEIGHT   = 12,              // Maximum image height (binning dependent)
    XSP_ROIX         = 13,              // Upper left X of ROI (pels)
    XSP_ROIY         = 14,              // Upper left Y of ROI (pels)
    XSP_ROIWIDTH     = 15,              // Width of ROI (pels)
    XSP_ROIHEIGHT    = 16,              // Height of ROI (pels)
    XSP_PERIOD       = 17,              // Period in ns (inverse of rate)
    XSP_PERIOD_MIN   = 18,              // Minimum period in ns (inverse of max rate)
    XSP_IRIG         = 19,              // Enables/disables IRIG
    XSP_SYNCOUT_CFG  = 20,              // Sync out cfg (see XS_SYNCOUT_CFG)
    XSP_TRIGIN_DEB   = 21,              // Trigger in debounce lenght [ns]
    XSP_TRIGIN_DEL   = 22,              // Trigger in delay [ns]
    XSP_SYNCOUT_WID  = 23,              // sync out width [ns]
    XSP_SYNCOUT_DEL  = 24,              // sync out delay [ns]
    XSP_SYNCIN_DEB   = 25,              // sync in debounce lenght [ns]
    XSP_SYNCIN_DEL   = 26,              // sync in delay [ns]
    XSP_HDMI_MODE    = 27,              // HDMI output mode (see XS_HDMI_MODE)
    XSP_PREV_MODE    = 28,              // preview mode (see XS_PREV_MODE)
    XSP_NOISE_RED    = 30,              // enable background correction (factory calibration file)
    XSP_NOISE_SENS   = 31,              // enable pixel sensitivity correction PSC (factory calibration file)
    XSP_NOISE_DKCOL  = 32,              // use dark columns to reduce noise
    XSP_NOISE_AUTO   = 33,              // enable current conditions background correction
    XSP_VIDEO_MODE   = 34,              // Video mode (see XS_VIDEO_MODE)
    XSP_NET_PERFORM  = 35,              // network performance
    XSP_PLUS         = 36,              // plus mode (double speed)
    XSP_GAMMA        = 37,              // gamma correction (0..30)
    XSP_FRAMES       = 38,              // total number of images
    XSP_PRE_TRIG     = 39,              // number of pre-trigger images
    XSP_BROC_LEN     = 40,              // BROC lenght
    XSP_FRAME_CAP    = 41,              // Frame capacity in current conditions (max number of frames)
    XSP_1ST_FRM_IDX  = 42,              // Index of first frame in memory
    XSP_STARTADDRLO  = 43,              // acquisition's start address (low part)
    XSP_STARTADDRHI  = 44,              // acquisition's start address (high part)
    XSP_NOISE_APSC   = 45,              // enable current conditions PSC
    XSP_EXT_PERIOD     = 46,            // External sync period [ns] (read-only)
    XSP_EXT_PULSE_WIDTH = 47,           // External sync pulse width [ns] (read-only)
    XSP_CMP_RATIO     = 48,             // compression ratio (40% to 100%, N3 camera only)
    XSP_FRAME_SIZE   = 49,              // frame size in camera memory [bytes]
    XSP_EXPOSURE_MAX = 50,              // max allowed exposure in current conditions [ns] (read-only)
    XSP_EXPOSURE_DBL = 51,              // value of second exposure in dbl exp mode
    XSP_XDR_RATIO    = 52,              // XDR ratio (2 to 8)
    XSP_XDR_CONTRAST = 53,              // XDR contrast (1 to 100)
    XSP_DYNAMIC_NR   = 54,              // dynamic noise reduction [0 to 30] dfl = camera dependent
    XSP_SHARPEN      = 55,              // sharpening [0 to 10] dfl = 0
    XSP_BRIGHTNESS   = 56,              // brightness [0 to 50] dfl = 25
    XSP_CONTRAST     = 57,              // contrast [0 to 20] dfl = 10
    XSP_HUE          = 58,              // hue [0 to 360] dfl = 180
    XSP_SATURATION   = 59,              // saturation [0 to 20] dfl = 10
    XSP_WB_11        = 60,              // White Balance Matrix value 1,1 (format 16.16) - Blue gain
    XSP_WB_12        = 61,              // White Balance Matrix value 1,2 (format 16.16)
    XSP_WB_13        = 62,              // White Balance Matrix value 1,3 (format 16.16)
    XSP_WB_21        = 63,              // White Balance Matrix value 2,1 (format 16.16)
    XSP_WB_22        = 64,              // White Balance Matrix value 2,2 (format 16.16) - Green Gain
    XSP_WB_23        = 65,              // White Balance Matrix value 2,3 (format 16.16)
    XSP_WB_31        = 66,              // White Balance Matrix value 3,1 (format 16.16)
    XSP_WB_32        = 67,              // White Balance Matrix value 3,2 (format 16.16)
    XSP_WB_33        = 68,              // White Balance Matrix value 3,3 (format 16.16) - Red Gain
    XSP_BOARD_TEMP   = 69,              // board temperature in hundreds of deg C (Y cameras only)
    XSP_MT_CFG       = 70,              // Motion Trigger mode (see XS_MTRIG_CFG)
    XSP_MT_THRESHOLD = 71,              // Motion Trigger threshold
    XSP_MT_ROIX      = 72,              // Upper left X of Motion Trigger ROI
    XSP_MT_ROIY      = 73,              // Upper left Y of Motion Trigger ROI
    XSP_MT_ROIWIDTH  = 74,              // Width of Motion Trigger ROI [pels]
    XSP_MT_ROIHEIGHT = 75,              // Height of Motion Trigger ROI [pels]
    XSP_DGR_SIZE     = 76,              // Configurable Datagram size (see XS_DGR_SIZE)
    XSP_AE_ENABLE    = 77,              // enable auto-exposure (0-1)
    XSP_AE_ROIX      = 78,              // Auto-exposure RO - X origin
    XSP_AE_ROIY      = 79,              // Auto-exposure RO - Y origin
    XSP_AE_ROIWIDTH  = 80,              // Auto-exposure RO - Width
    XSP_AE_ROIHEIGHT = 81,              // Auto-exposure RO - Height
    XSP_AE_REFERENCE = 82,              // luminance reference [20..200]
    XSP_AE_SPEED     = 83,              // Auto-exposure speed (reaction to intensity change)
    XSP_AE_EXPOSURE  = 84,              // read the current value of exposure (read-only)
    XSP_PROP         = 85,              // Post recording operation (see XS_PR_OP);
    XSP_PROP_WR_BLK  = 86,              // Post rec Op: blocks of 512 bytes to write in the device (SD card, Net disk)
    XSP_PROP_ABORT   = 87,              // Post rec Op: if the value is non 0, the Post-Rec operation is aborted.
    XSP_SYNCOUT_ALIGN  = 88,            // sync out alignment (see XS_SYNCOUT_ALIGN)
    XSP_MARKER_CFG     = 89,            // enable marker (see XS_MARKER_CFG)
    XSP_MARKER_VAL     = 90,            // return current marker value (0 or 1)
    XSP_CI_THR         = 91,            // color interpolation threshold [0 to 255], used if the CI mode is Advanced
    XSP_CLOCK_SPEED    = 92,            // clock speed read-only (see XS_CLOCK_SPEED)
    XSP_HD_ROI         = 93,            // HD ROI index for HD cameras (see XS_HD_ROI)
    XSP_GAUSS_FLT      = 94,            // Gaussian Filter [0 to 16]
    XSP_ROT_ANGLE      = 95,            // Rotation angle (see XS_ROT_ANGLE)
    XSP_FLIP           = 96,            // Image flip (see XS_FLIP)
    XSP_LUT            = 97,            // Lookup Table (see XS_LUT)
    XSP_LUT_CHN_MASK   = 98,            // Lookup Table RGB channels mask (see XS_LUT_MASK)
    XSP_HDMI_OVERLAY   = 99,            // Enable HDMI overlay (0=Disable, 1=Enable)
    XSP_WBTBL_TEMP     = 100,           // color balance table temperature (for display on HDMI)
    XSP_HD_ZOOM        = 101,           // digital zoom for HD cameras (see XS_HD_ZOOM)
    XSP_HD_ZOOM_X      = 102,           // X coord of origin in digital zoom
    XSP_HD_ZOOM_Y      = 103,           // Y coord of origin in digital zoom
    XSP_SHARPEN_THR    = 104,           // sharpening threshold
    XSP_SYNCIN_INV     = 105,           // Sync in polarity inversion
    XSP_DYNAMIC_NR2    = 106,           // additional dynamic noise reduction [0 to 30]
    XSP_JPEG           = 107,           // Disable/Enable JPEG (uncompressed and compressed)
    XSP_IMG_ZOOM       = 108,           // image zoom (resize image factor)
    XSP_IMG_ZOOM_WID   = 109,           // image width after zoom (read only)
    XSP_IMG_ZOOM_HGT   = 110,           // image height after zoom (read only)
    XSP_AE_CUR_LUMA    = 111,           // Calculates current luma in AE window (read only - the camera must be in rec mode)
    XSP_BROC_TOT_LEN   = 112,           // Broc total number of frames
    XSP_BROC_CURR_SECT = 113,           // Current recording BROC section index (read-only)
    XSP_HD_ZOOM_2R     = 114,           // Disable magnification and take ROI
    XSP_FRAME_SIZE_IMG = 115,           // Size of a frame if saved to disk as image (read-only)
    XSP_SYNC_COUNT     = 116,           // total number of sync counts in latest acquisition
    XSP_AE_MIN_EXP     = 117,           // set the value of min exposure in Auto-Exposure mode
    XSP_AE_MAX_EXP     = 118,           // set the value of max exposure in Auto-Exposure mode
    XSP_BATTERY_STATUS = 119,           // battery status (see XS_BATTERY, read-only, NX cameras only)
    XSP_JPEG_QUAL      = 120,           // jpeg quality (1 to 100, dfl 90)
    XSP_SSD_STRM_PER   = 121,           // inverse of maximum streaming speed to SSD
    XSP_SSD_MAX_FRMS   = 122,           // max number of frames that can be stored in SSD without overrun
    XSP_GAMMA_LEVEL    = 123,           // gamma level 2 (multi-gamma)
    XSP_GAMMA1         = 124,           // gamma value 1 (multi-gamma)
    XSP_GAMMA_LEVEL1   = 125,           // gamma level 1 (multi-gamma)
    XSP_MISS_STEP_CNT  = 126,           // Number of mission steps that have been configured (readonly)
    XSP_EXEC_STEP_CNT  = 127,           // Number of mission steps that have been executed (readonly)
    XSP_EXT_FREQ       = 128,           // External sync frequency [Hz] (read-only)
    XSP_SSD_WRITE_PERC = 129,           // Percentage of data written in the SSD
    XSP_RAW_RD_TSTAMP  = 130,           // Enable fast read of time stamp from RAW data
    XSP_NET_SPEED      = 131,           // Network speed for HG (0:automatic, 1:100Mbps, 2:1000 Mbps)
    XSP_AE_EXP_STEP    = 132,           // Auto-exposure step size (exposure change)
    XSP_SYNCIN_JITTER  = 133,           // sync in jitter in external pll mode
    XSP_SYNCIN_DIV     = 134,           // sync in period divider/freq multiplier in external pll mode
    XSP_IRIG_GPS_JITT  = 135,           // jitter tolarance for IRIG/GPS
    XSP_LENS_FOCUS     = 136,           // lens focus position
    XSP_LENS_IRIS      = 137,           // lens iris/aperture position
    XSP_RAW_MIN_IDX    = 138,           // index of first frame in a raw sequence
    XSP_SHOCK_THR      = 139,           // Impact sensor shock threshold level (in multiples of 0.016 g)
    XSP_SHOCK_TIME     = 140,           // Impact sensor shock maximum duration (in multiples of 744 us)
    XSP_LATEST_FRM_IDX = 141,           // index of the latest acquired frame (read-only)
    XSP_LENS_ZOOM      = 142,           // lens zoom position
    XSP_PAINT_GAIN_R   = 143,           // Paint parameter: Gain - R component
    XSP_PAINT_GAIN_G   = 144,           // Paint parameter: Gain - G component
    XSP_PAINT_GAIN_B   = 145,           // Paint parameter: Gain - B component
    XSP_PAINT_OFFS_R   = 146,           // Paint parameter: Offset/Pedestal - R component
    XSP_PAINT_OFFS_G   = 147,           // Paint parameter: Offset/Pedestal - G component
    XSP_PAINT_OFFS_B   = 148,           // Paint parameter: Offset/Pedestal - B component
    XSP_DEFLICKER      = 149,           // deflicker
    XSP_PTP_MODE       = 150,           // PTP mode (0:ethernet 1:UDP)
    XSP_DEFLICKER_THR  = 151,           // deflicker threshold
    XSP_PTP_DELAY      = 152,           // PTP power up delay [s]
    XSP_LENS_FOCUS_REL = 153,           // Lens focus delta
    XSP_LENS_CMD       = 154,           // Lens command (see XS_LENS_CMD list)
} XS_PARAM;

typedef enum
{
    XS_ATTR_MIN      = 0,            // minimum value of the parameter
    XS_ATTR_MAX      = 1,            // maximum value of the parameter
    XS_ATTR_READONLY = 2,            // returns if the parameter is read-only
    XS_ATTR_DEFAULT  = 3,            // the default value of the parameter
} XS_ATTRIBUTE;

typedef enum
{
    // general
    XSI_CAMERA_MODEL        = 0,     // Camera Model (see XS_CAM_MODEL)
    XSI_CAMERA_ID           = 1,     // Camera ID (see XS_ENUMITEM)
    XSI_FW_VERSION          = 2,     // EEPROM Firmware version
    XSI_MEMORY              = 3,     // Amount of DDR memory installed on the camera [bytes]
    XSI_REC_SIZE_SELECTABLE = 4,     // User can select amount of memory to be recorded
    XSI_COOLED              = 5,     // the camera is cooled
    XSI_SERIAL              = 6,     // serial number
    XSI_REVISION            = 7,     // camera revision (A, B, C, etc.)
    XSI_DACS_COUNT          = 8,     // number of Digital to Analog Converters
    XSI_INT_CLOCK           = 9,     // internal clock rate in Hz
    XSI_EXTRA_ROWS          = 10,    // extra rows in read-out
    XSI_ROW_CLOCKS          = 11,    // number of clocks for each row readout
    XSI_SNS_TYPE            = 12,    // Sensor type (see XS_SNS_TYPE)
    XSI_SNS_MODEL           = 13,    // Sensor model (see XS_SNS_MODEL)
    XSI_SNS_WIDTH           = 14,    // Sensor Maximum width [pixels]
    XSI_SNS_HEIGHT          = 15,    // sensor Maximum height [pixels]
    XSI_CFW_VERSION         = 16,    // Controller Firmware Version
    XSI_LIVE_WHILE_REC      = 17,    // the camera supports Live while record
    XSI_SOFT_TRIGGER        = 18,    // the camera supports software trigger
    XSI_FCAL_FILE_ON        = 19,    // the factory calibration file is installed on the hard disk
    XSI_IRIG                = 20,    // the camera supports IRIG
    XSI_BNC_CONNECTORS      = 21,    // the number of BNC connectors on camera (2 or 3)
    XSI_EXP_STEPS           = 22,    // number of exposure steps
    XSI_INTENSIFIED         = 23,    // the camera is intensified
    XSI_LINK                = 24,    // the camera link (USB 2.0, GEthernet)
    XSI_MIN_EXP             = 25,    // minimum exposure (1 us or 100 ns)
    XSI_MISC_CAPS           = 26,    // miscellaneous capabilities
    XSI_CFW_CLK_1           = 27,    // controller firmware clock 1 value (deprecated)
    XSI_INT_REG_0           = 28,    // camera internal register 0 (model-dependent)
    XSI_INT_REG_1           = 29,    // camera internal register 1 (model-dependent)
    XSI_LIGHT               = 30,    // the camera is light (reduced capabilities)
    XSI_GIGA_ETHERNET       = 31,    // the camera has Giga-ethernet support
    XSI_FLASH_MEMORY        = 32,    // internal flash memory size (MB)
    XSI_VIDEO_MODE          = 33,    // the camera supports video mode
    XSI_PLUS                = 34,    // the camera supports the plus mode (double speed)
    XSI_CAMERA_NAME         = 35,    // camera name (string)
    XSI_LINK_FW_VER         = 36,    // Link (USB/GE firmware version)
    XSI_CFW2_VERSION        = 37,    // Second Controller Firmware Version
    XSI_ROI_MIN             = 38,    // Minimum allowed ROI (X, Y)
    XSI_ROI_STEP            = 39,    // ROI Step size (X,Y)
    XSI_FG_TYPE             = 40,    // Frame grabber type (see XS_FG_TYPE)
    XSI_CCAL_FILE_ON        = 41,    // the calibration file with current conditions is installed on the hard disk
    XSI_FCAL_FILE_FLASH     = 42,    // the factory calibration file is stored in camera flash memory
    XSI_CAL_PATH            = 43,    // path to the camera calibration files folder (directory)
    XSI_CFA_PATTERN         = 44,    // CFA Bayer pattern of color sensor (see XS_CFA_PATTERN)
    XSI_MAX_FRM_SIZE        = 45,    // Maximum frame size in camera memory (or computer) [bytes]
    XSI_NEW_DESIGN          = 46,    // The camera hardware is redesigned (with internal image processing)
    XSI_CAL_NAME            = 47,    // the camera calibration file name
    XSI_MOTION_TRIG         = 48,    // the camera supports motion trigger
    XSI_WRITE_2_DISK        = 49,    // the camera supports direct write to disk (M2D only)
    XSI_COMPRESSION         = 50,    // the camera supports data compression (N cameras)
    XSI_DGR_SIZE            = 51,    // the camera supports configurable datagram size
    XSI_AUTO_EXPOSURE       = 52,    // the camera supports the auto exposure
    XSI_POST_REC_OP         = 53,    // the camera supports post-recording operation (write to SD card)
    XSI_MARKER              = 54,    // the camera supports the marker option
    XSI_SUB_MODEL           = 55,    // the camera sub-model
    XSI_SNS_PIX_DEPTH       = 56,    // the sensor pixel depth (12,10 or 8)
    XSI_XDR_SUPPORT         = 57,    // the camera supports XDR
    XSI_EDR_SUPPORT         = 58,    // the camera supports EDR
    XSI_LIVE_BUF_SIZE       = 59,    // minimum buffer in camera memory to avoid overwrite of acquisition during live
    XSI_RESIZE              = 60,    // the camera supports image resize for fast thumbnail live.
    XSI_HW_BOARDS           = 61,    // number of boards in hardware configuration (0 if not supported)
    XSI_HW_INFO             = 62,    // information about boards of Hw configuration ()
    XSI_TNR_SUPPORT         = 63,    // the camera supports TNR
    XSI_PIV_READY           = 64,    // the camera is PIV ready (timing signals on the status connector)
    XSI_CUR_CAL             = 65,    // the camera supports the current conditions calibration procedure
    XSI_JPEG_SUPPORT        = 66,    // the camera supports JPEG compression on images
    XSI_DNR2_SUPPORT        = 67,    // the camera support TNR and DNR together
    XSI_HW_BROC_SUPPORT     = 68,    // the camera support HW BROC
    XSI_ASYNC_VIDEO_PB      = 69,    // the camera supports asynchronous video (HDMI) playback
    XSI_EX_ROI_CNT          = 70,    // number of extended ROI the camera supports
    XSI_EX_ROI_MAX_WID      = 71,    // extended ROI maximum width
    XSI_EX_ROI_MAX_HGT      = 72,    // extended ROI maximum height
    XSI_1PPS_SYNC_SUPPORT   = 73,    // the camera suports the 1 PPS on sync in and sync out
    XSI_BATTERYSTS_SUPPORT  = 74,    // the camera supports battery status and level readout
    XSI_SSD_SIZE            = 75,    // size of onboard SSD
    XSI_FAST_LIVE           = 76,    // the camera supports the XsLive routine for fast live display
    XSI_PTP_SUPPORT         = 77,    // the camera supports PTP
    XSI_TEMP_SUPPORT        = 78,    // the camera supports temperature read [hundreds of deg C]
    XSI_CI_THR_SUPPORT      = 79,    // the camera supports the Color Interpolation threshold value
    XSI_SI_PLL_SUPPORT      = 80,    // the camera supports Phase lock loop sync in
    XSI_SI_DPULSE_SUPPORT   = 81,    // the camera supports dynamic pulse sync in
    XSI_LENS_SUPPORT        = 82,    // the camera firmware supports motorized lens control
    XSI_BATTERY_INFO        = 83,    // battery information (manufacturing date)
    XSI_SHOCK_SNS_SUPPORT   = 84,    // impact shock sensor
    XSI_PAINT_SUPPORT       = 85,    // the camera supports paint parameters (gain/offset)
    XSI_DEFLICKER_SUPPORT   = 86,    // the camera supports deflicker
    XSI_PTP_MODE_SUPPORT    = 87,    // the camera supports PTP mode configuration
    XSI_LENS_INFO           = 88,    // lens information

    // USB specific
    XSI_USB_VID             = 100,   // USB vendor ID
    XSI_USB_PID             = 101,   // USB product ID
    XSI_USB_PORT            = 102,   // USB port number

    // Gigabit Ethernet specific
    XSI_GE_CAM_MACADD       = 200,   // camera MAC address
    XSI_GE_CAM_IPADD        = 201,   // camera IP address
    XSI_GE_ADP_MACADD       = 202,   // adapter MAC address
    XSI_GE_ADP_IPADD        = 203,   // adapter IP address
    XSI_GE_ADP_NETMASK      = 204,   // adapter network mask
    XSI_GE_CAM_NETMASK      = 205,   // camera network mask
    XSI_GE_ADP_MTU          = 206,   // adapter MTU

} XS_INFO;

//////////////////////////////////////////////////////////////////
// Data structures and types

// camera object handle
typedef void* XS_HANDLE;

// Camera Enumeration Item
typedef struct _XS_ENUMITEM
{
    XSULONG32 cbSize;         // size of this structure

    // general
    XSULONG32 nCameraId;      // camera ID
    XSULONG32 nCameraModel;   // camera model (XS-3, etc.)
    XSULONG32 nSensorType;    // sensor type (monochrome/color)
    XSULONG32 nSensorModel;   // sensor model (XS-3, etc.)
    XSULONG32 nSerial;        // serial number
    XSULONG32 nRevision;      // camera revision
    XSULONG32 nEFWVersion;    // EEPROM Firmware version
    XSULONG32 nCFWVersion;    // controller Firmware version
    XSULONG32 bIsOpen;        // 1 if already open
    XSULONG32 nLinkType;      // link type (USB 2.0, Giga-Ethernet, Camera-Link)
    XSULONG32 bIRIG;          // the camera has IRIG
    XSULONG32 bIntensified;   // the camera is intensified
    XSULONG32 nIntClock;      // internal clock frequency [Hz]
    XSULONG32 nLicInfo;       // License information
    XSULONG32 nSubModel;      // camera sub-model
    XSULONG32 bEDR;           // the camera supports EDR
    XSULONG32 nMinExp;        // minimum allowed exposure in ns (1000 or 100)
    XSULONG32 nGeAdpMTU;      // Network Adapter maximum transmission unit
    XSULONG32 nMiscCaps;      // miscellaneous capabilities (see XS_MISC_CAPS)
    XSULONG32 bDgrSize;       // the camera supports configurable datagram size (jumbo packets)
    XSULONG32 bNewDesign;     // the camera is redesigned (Y with on-board image processing)
    XSULONG32 bLight;         // the camera is alight version (reduced capabilities)
    XSULONG32 bPlus;          // the camera is a plus model (double speed)
    XSULONG32 bGigaEth;       // the camera has Giga-ethernet link (and USB 2.0)

    // USB-specific
    XSULONG32 nUsbVID;        // Vendor ID
    XSULONG32 nUsbPID;        // Product ID
    XSULONG32 nUsbPort;       // USB port number

    // gigabit ethernet specific
    XSULONG32 nGeCamMACAddLo;      // Camera MAC address (low part)
    XSULONG32 nGeCamMACAddHi;      // Camera MAC address (high part)
    XSULONG32 nGeCamIPAdd;         // Camera IP address
    XSULONG32 nGeCamNetMask;       // Camera subnet mask
    XSULONG32 nGeCamDfltGw;        // Camera default gateway
    XSULONG32 nGeCamCmdPort;       // Camera Command UDP Port
    XSULONG32 nGeAdpMACAddLo;      // Adapter MAC address (low part)
    XSULONG32 nGeAdpMACAddHi;      // Adapter MAC address (high part)
    XSULONG32 nGeAdpIPAdd;         // Adapter IP address
    XSULONG32 nGeAdpNetMask;       // Adapter subnet mask

    char szCameraName[L_CAMNAME];  // camera name
    XSULONG32 nCFW2Version;        // second controller Firmware version

    // camera link specific
    XSULONG32 nFGType;             // frame grabber type (see XS_FG_TYPE)

    XSULONG32 nCFW3Version;        // controller Firmware version 3 (build number)
    XSULONG32 nCFAPattern;         // Color filter array pattern (see XS_CFA_PATTERN)
    XSULONG32 nSSDSizeLo;          // SSD size (lo word)
    XSULONG32 nSSDSizeHi;          // SSD size (hi word)
    XSULONG32 nBatteryInfo;        // Battery info (manufacturing date+SN)
    XSULONG32 nDDRSizeLo;          // DDR (memory) size in bytes (low part)
    XSULONG32 nDDRSizeHi;          // DDR (memory) size in bytes (high part)
    XSULONG32 anFill[10];          // fill parameters, for future use

} XS_ENUMITEM, *PXS_ENUMITEM;


// Camera opaque configuration structure
typedef struct _XS_SETTINGS
{
    XSULONG32 cbSize;          // size of this structure
    XSULONG32 nData[255];      // reserved data array
} XS_SETTINGS, *PXS_SETTINGS;

//
// For calls to Grab routines
//
// These fields must be filled before the Api call:
//     pBuffer
//     nBufSize
//     nImages
//
// These fields are set by the dll during the Api call:
//     nFormat
//     nWidth
//     nHeight
//     nBitDepth
//     nErrorCode
//
typedef struct
{
    void*     pBuffer;          // Image buffer, 4-byte aligned
    XSULONG32 nBufSize;         // Length of buffer, in BYTES
    XSULONG32 nImages;          // 1 in single exposure, 2 in double exposure

    XSULONG32 nFormat;          // Image Format
    XSULONG32 nWidth;           // Image width [pels]
    XSULONG32 nHeight;          // Image height [pels]
    XSULONG32 nPixDepth;        // Pixel depth
    XSULONG32 nErrorCode;       // XS_ERROR code for this frame

} XS_FRAME, *PXS_FRAME;

// XsGetBrocparameters BROC section
typedef struct _XS_BROC_SECTION
{
    XSULONG32 nStartAddrLo;
    XSULONG32 nStartAddrHi;
    XSULONG32 n1stFrmIdx;
    XSULONG32 nTrgTime;

} XS_BROC_SECTION, *PXS_BROC_SECTION;

// BROC array of sections
typedef struct _XS_BROC
{
    XS_BROC_SECTION sect[256];
} XS_BROC, *PXS_BROC;

// GPS/IRIG structure
typedef struct _XS_GPSTIMING
{
    XSULONG32 nSignalPresent;
    XSULONG32 nYear;
    XSULONG32 nDayOfYear;
    XSULONG32 nHours;
    XSULONG32 nMinutes;
    XSULONG32 nSeconds;
    XSULONG32 nMicroSeconds;
    XSULONG32 nFlags;
} XS_GPSTIMING, *PXS_GPSTIMING;

// Write To Disk Configuration
typedef struct _XS_W2DCFG
{
    XSULONG32 nDDRBufSize;  // size of DDR circular buffer (M-only)
    XSULONG32 nOpt;         // write to disk options
    XSULONG32 nDrives;      // number of drives/volumes (1,2 or 4)
    char szVolume1[64];     // volume 1 (drive in Windows)
    char szVolume2[64];     // volume 2 (drive in Windows)
    char szVolume3[64];     // volume 3 (drive in Windows)
    char szVolume4[64];     // volume 4 (drive in Windows)
    char szDirectory[512];  // directory path (to be added to each volume)
} XS_W2DCFG, *PXS_W2DCFG;

///////////////////////////////////////////////////////////////////////////////

#ifdef __cplusplus
extern "C" {
#endif

///////////////////////////////////////////////////////////////////////////////
// Callback prototype

typedef void ( XSTREAMAPI *XS_AsyncCallback )
(
    void*     pUserData,      // User defined variable
    XS_ERROR  nErrCode,       // Error code
    XSULONG32 nFlags          // Combination of XS_CALLBACK_FLAGS
);

typedef void ( XSTREAMAPI *XS_ProgressCallback )
(
    void*     pUserData,      // User defined variable
    XSULONG32 nProgress,      // Current progress count
    XSULONG32 nCount          // Total progess count
);

// announcements callback
typedef void ( XSTREAMAPI *XS_AnnouncementCallback )
(
    void *pUserData,          // User defined variable
    char *pszAnnouncement     // Announcements string
);

// stream to disk callback
typedef void ( XSTREAMAPI *XS_StreamingCallback )
(
    void*     pUserData,     // User defined variable
    XSULONG32 nParam1,       // Total number of frames saved
    XSULONG32 nParam2,       // percentage of available ddr buffer (0 to 1000)
    XSULONG32 nErrCode
);

///////////////////////////////////////////////////////////////////////////////
// Function prototypes

//*****************************************************************************
//*
//*  XsGetVersion
//*
//* Descr: Get the version of this module (the XstreamVISION Driver).
//*
//* Param: pVersionMS: the most significant 32 bit of the version number
//*        pVersionLS: the least significant 32 bit of the version number
//*        pIsDemo: if the driver is demo the returned value is 1, otherwise 0
//*
//* Ret:   XS_SUCCESS: version numbers are valid
//*        XS_E_GENERIC_ERROR: Could not extract version numbers from the driver.
//*
//*****************************************************************************
//
XS_ERROR XSTREAMAPI XsGetVersion(PXSULONG32 pVersionMS,
                                 PXSULONG32 pVersionLS,
                                 PXSULONG32 pIsDemo);

//*****************************************************************************
//*
//*  XsLoadDriver
//*
//* Descr: Loads the camera driver
//*
//* Param: nUSBNotify: activates or deactivates the USB notification.
//*
//* Ret:   XS_SUCCESS: the driver is successfully loaded
//*        XS_E_GENERIC_ERROR: could not load the driver
//*
//*****************************************************************************
//
XS_ERROR XSTREAMAPI XsLoadDriver(unsigned short nUSBNotify);

//*****************************************************************************
//*
//*  XsUnloadDriver
//*
//* Descr: Closes the Driver and releases any allocated resource.
//*
//* Param: None
//*
//* Ret:   None
//*
//*****************************************************************************
//
void XSTREAMAPI XsUnloadDriver(void);

//*****************************************************************************
//*
//*  XsEnumCameras
//*
//* Descr: Lists all the available cameras.
//*
//* Param: pItemList: pointer to enumerate sructure array
//*        pItemNr: [in]  pointer to the number of structures in the array
//*                 [out] pointer to number of cameras detected
//*        nEnumFilter: enumeration filter (see XS_ENUM_FLT).
//*
//* Ret:   XS_SUCCESS: a valid list is returned
//*        XS_E_HARDWARE_FAULT: error using driver (SEVERE error)
//*
//*****************************************************************************
//
XS_ERROR XSTREAMAPI XsEnumCameras(PXS_ENUMITEM pItemList,
                                  PXSULONG32 pItemNr,
                                  XSULONG32 nEnumFilter);


//*****************************************************************************
//*
//*  XsPreConfigCamera
//*
//* Descr: configures a camera parameter before the camera is open or configures
//*        a system parameter
//*
//* Param: nCameraId - camera ID
//*        nParamKey - parameter index (see XS_PRE_PARAM)
//*        nValueLo - the value to write (LS part)
//*        nValueHi - the value to write (MS part)
//*
//* Ret:   XS_SUCCESS - a valid handle is returned
//*        XS_E_INVALID_CAMERA_ID - invalid camera id
//*        XS_E_CAM_ALREADY_OPEN - the camera is already open
//*        XS_E_NOT_SUPPORTED - not supported nParamKey
//*
//*****************************************************************************
//
XS_ERROR XSTREAMAPI XsPreConfigCamera(void *nCameraId,
                                      XS_PRE_PARAM nParamKey,
                                      void *nValueLo,
                                      void *nValueHi);

//*****************************************************************************
//*
//*  XsOpenCamera
//*
//* Descr: opens the camera with the nCameraId ID and returns the handle to that
//*        camera. The available cameras ID can be retrieved by using the
//*        XsEnumCameras routine. Once the camera is open, use the returned handle
//*        for any operation.
//*
//* Param: nCameraId - camera ID
//*        pHandle - pointer to the handle variable which receives the camera handle
//*                  use it in any further camera operation.
//*
//* Ret:   XS_SUCCESS - a valid handle is returned
//*        XS_E_INVALID_CAMERA_ID - invalid camera id
//*        XS_E_CAM_ALREADY_OPEN - the camera is already open
//*        XS_E_HARDWARE_FAULT: error using driver (SEVERE error)
//*
//*****************************************************************************
//
XS_ERROR XSTREAMAPI XsOpenCamera(XSULONG32 nCameraId,
                                 XS_HANDLE* pHandle);

//*****************************************************************************
//*
//*  XsOpenRawCamera
//*
//* Descr: opens the raw file as a virtual camera and returns the handle
//*
//* Param: lpszRawFielPath - full path to the rawfile.xml or to the directory that
//                           contains the file
//                           (ex: c:/Images/acq01 or c:/images/acq01/rawfile.xml)
//*        pHandle - pointer to the variable which receives the virtual camera handle
//*
//* Ret:   XS_SUCCESS - a valid handle is returned
//*        XS_E_INVALID_CAMERA_ID - invalid path
//*        XS_E_CAM_ALREADY_OPEN - the camera is already open
//*        XS_E_HARDWARE_FAULT: error using driver (SEVERE error)
//*
//*****************************************************************************
//
XS_ERROR XSTREAMAPI XsOpenRawCamera(const char *lpszRawFilePath,
                                    XS_HANDLE* pHandle);

//*****************************************************************************
//*
//*  XsCloseCamera
//*
//* Descr: closes a camera previously open. After that the handle is no longer
//*        valid.
//*
//* Param: hCamera - hadle to an open camera.
//*
//* Ret:   XS_SUCCESS - camera successfully open.
//*        XS_E_INVALID_HANDLE - invalid camera handle
//*
//*****************************************************************************
//
XS_ERROR XSTREAMAPI XsCloseCamera(XS_HANDLE hCamera);

//*****************************************************************************
//*
//*  XsGetCameraInfo
//*
//* Descr: gets a camera information value
//*
//* Param: hCamera - handle to an open camera
//*        nInfoKey - index of the information item (see XS_INFO values)
//*        pValue - ptr to the variable which receives the value.
//*
//* Ret:   XS_SUCCESS - value successfully read.
//*        XS_E_INVALID_HANDLE - invalid camera handle
//*        XS_E_NOT_SUPPORTED - not supported info key value
//*
//*****************************************************************************
//
XS_ERROR XSTREAMAPI XsGetCameraInfo(XS_HANDLE hCamera,
                                    XS_INFO nInfoKey,
                                    PXSULONG32 pValueLo,
                                    PXSULONG32 pValueHi);

//*****************************************************************************
//*
//*  XsSetCameraInfo
//*
//* Descr: sets a camera information value (not all the items can be set)
//*
//* Param: hCamera - handle to an open camera
//*        nInfoKey - index of the information item (see XS_INFO values)
//*        nValueLo,nValueHi - the value to set.
//*
//* Ret:   XS_SUCCESS - value successfully written.
//*        XS_E_INVALID_HANDLE - invalid camera handle
//*        XS_E_NOT_SUPPORTED - not supported info key value
//*        XS_E_INVALID_VALUE - the info values are not valid
//*        XS_E_INVALID_ARGUMENTS - invalid function arguments
//*
//*****************************************************************************
//
XS_ERROR XSTREAMAPI XsSetCameraInfo(XS_HANDLE hCamera,
                                    XS_INFO nInfoKey,
                                    XSULONG32 nValueLo,
                                    XSULONG32 nValueHi);

////////////////////////////////////////////////////////////////////////////////////////////////////
// Camera configuration routines
// the dll can read the parameter from the camera or default and fill the XS_SETTINGS structure.
// the parameters values can be read or written by calling the getParameter or SetParameter routines
// Then the routine RefreshCameraSettings must be called to make the changes effective in the camera
////////////////////////////////////////////////////////////////////////////////////////////////////

//*****************************************************************************
//*
//*  XsReadDefaultSettings
//*
//* Descr: Reads the default settings from the camera and fills up the XS_SETTINGS
//*        structure
//*
//* Param: hCamera - handle to an open camera
//*        pCfg - pointer to a XS_SETTINGS structure
//*
//* Ret:   XS_SUCCESS - default successfully read.
//*        XS_E_INVALID_HANDLE - invalid camera handle
//*        XS_E_INVALID_CFG - invalid XS_SETTINGS struct. Make sure that the
//*                           cbSize field has been set.
//*
//*****************************************************************************
//
XS_ERROR XSTREAMAPI XsReadDefaultSettings(XS_HANDLE hCamera,
                                          PXS_SETTINGS pCfg);

//*****************************************************************************
//*
//*  XsReadCameraSettings
//*
//* Descr: Reads the current camera configuration structure
//*
//* Param: hCamera - handle to an open camera
//*        pCfg - pointer to a XS_SETTINGS structure
//*
//* Ret:   XS_SUCCESS - configuration successfully read.
//*        XS_E_INVALID_HANDLE - invalid camera handle
//*        XS_E_INVALID_CFG - invalid XS_SETTINGS struct. Make sure that the
//*                           cbSize field has been set.
//*
//*****************************************************************************
//
XS_ERROR XSTREAMAPI XsReadCameraSettings(XS_HANDLE hCamera,
                                         PXS_SETTINGS pCfg);

//*****************************************************************************
//*
//*  XsRefreshCameraSettings
//*
//* Descr: Send the configuration structure to the camera All the changes to
//*        the XS_SETTINGS structure (XsSetParameter) will take effect after
//*        this routine is called
//*
//* Param: hCamera - handle to an open camera
//*        pCfg - pointer to a XS_SETTINGS structure
//*
//* Ret:   XS_SUCCESS - configuration successfully refreshed on the camera.
//*        XS_E_INVALID_HANDLE - invalid camera handle
//*        XS_E_INVALID_CFG - invalid XS_SETTINGS struct. Make sure that the
//*                           cbSize field has been set.
//*        XS_E_BUSY - the operation cannot be completed because the cmera is busy
//*
//*****************************************************************************
//
XS_ERROR XSTREAMAPI XsRefreshCameraSettings(XS_HANDLE hCamera,
                                            PXS_SETTINGS pCfg);

//*****************************************************************************
//*
//*  XsQueueCameraSettings
//*
//* Descr: Queues the configuration. This function returns immediately.
//*        When the camera configuration has changed, a callback is called.
//*
//* Param: hCamera - handle to an open camera
//*        pCfg - pointer to a XS_SETTINGS structure
//*        pfnCallback - callback routine pointer, it may be NULL
//*        nFlags - callback flags
//*        pUserData - parameter returned in callback, it may be a pointer
//*                    to user data.
//*
//* Ret:   XS_SUCCESS - configuration successfully queued.
//*        XS_E_INVALID_HANDLE - invalid camera handle
//*        XS_E_INVALID_CFG - invalid XS_SETTINGS struct. Make sure that the
//*                           cbSize field has been set.
//*        XS_E_QUEUE_FULL - nothing more can be added to the queue.
//*        XS_E_HARDWARE_FAULT - error using driver (SEVERE error)
//*
//*****************************************************************************
//
XS_ERROR XSTREAMAPI XsQueueCameraSettings(XS_HANDLE hCamera,
                                          PXS_SETTINGS pCfg,
                                          XS_AsyncCallback pfnCallback,
                                          XSULONG32 nFlags,
                                          void *pUserData);

//*****************************************************************************
//*
//*  XsValidateCameraSettings
//*
//* Descr: Validates camera settings. Not all the combinations of parameters
//*        values are allowed. The routine fixes the structure.
//*
//* Param: hCamera - handle to an open camera
//*        pCfg - pointer to a XS_SETTINGS structure
//*
//* Ret:   XS_SUCCESS - configuration successfully validated.
//*        XS_E_INVALID_HANDLE - invalid camera handle
//*        XS_E_INVALID_CFG - invalid XS_SETTINGS struct. Make sure that the
//*                           cbSize field has been set.
//*
//*****************************************************************************
//
XS_ERROR XSTREAMAPI XsValidateCameraSettings(XS_HANDLE hCamera,
                                             PXS_SETTINGS pCfg);

//*****************************************************************************
//*
//*  XsReadSettingsFromFlash
//*
//* Descr: Read camera settings from camera flash memory, if available
//*
//* Param: hCamera - handle to an open camera
//*        pCfg - pointer to a XS_SETTINGS structure
//*
//* Ret:   XS_SUCCESS - configuration successfully validated.
//*        XS_E_NOT_SUPPORTED - not supported or not flash memory
//*        XS_E_INVALID_HANDLE - invalid camera handle
//*        XS_E_INVALID_ARGUMENTS - invalid funcion arguments
//*        XS_E_INVALID_CFG - invalid XS_SETTINGS struct. Make sure that the
//*                           cbSize field has been set.
//*        XS_E_NOT_IN_FLASH - the configuration is not stored in flash memory
//*
//*****************************************************************************
//
XS_ERROR XSTREAMAPI XsReadSettingsFromFlash(XS_HANDLE hCamera,
                                            PXS_SETTINGS pCfg);

//*****************************************************************************
//*
//*  XsWriteSettingsToFlash
//*
//* Descr: Writes camera settings to camera flash memory, if available
//*
//* Param: hCamera - handle to an open camera
//*        pCfg - pointer to a XS_SETTINGS structure
//*
//* Ret:   XS_SUCCESS - configuration successfully validated.
//*        XS_E_NOT_SUPPORTED - not supported or not flash memory
//*        XS_E_INVALID_HANDLE - invalid camera handle
//*        XS_E_INVALID_ARGUMENTS - invalid funcion arguments
//*        XS_E_INVALID_CFG - invalid XS_SETTINGS struct. Make sure that the
//*                           cbSize field has been set.
//*
//*****************************************************************************
//
XS_ERROR XSTREAMAPI XsWriteSettingsToFlash(XS_HANDLE hCamera,
                                           PXS_SETTINGS pCfg);

//*****************************************************************************
//*
//*  XsReadCameraSettingsArray
//*
//* Descr: Reads the current camera configuration structure
//*
//* Param: hCamera - handle to an open camera
//*        nOption - 0:from camera memory, 1:from SSD
//*        pCfgList - pointer to a XS_SETTINGS structure array
//*        pBrocList - pointer to a XS_BROC structure array (it may be NULL)
//*        pCfgCnt: [in]  pointer to the number of structures in the array
//*                 [out] pointer to number of configurations read
//*
//* Ret:   XS_SUCCESS - configuration array successfully read.
//*        XS_E_INVALID_HANDLE - invalid camera handle
//*        XS_E_INVALID_ARGUMENTS - invalid funcion arguments
//*        XS_E_NOT_SUPPORTED - the routine is not supported
//*
//*****************************************************************************
//
XS_ERROR XSTREAMAPI XsReadCameraSettingsArray(XS_HANDLE hCamera,
                                              XSULONG32 nOption,
                                              PXS_SETTINGS pCfgList,
                                              PXS_BROC pBrocList,
                                              PXSULONG32 pnCfgCnt);

//*****************************************************************************
//*
//*  XsGetParameterAttribute
//*
//* Descr: Gets a parameter attribute (min value, max value or read-only flag)
//*        
//* Param: hCamera - handle to an open camera
//*        pCfg - pointer to a valid XS_SETTINGS structure
//*        nParamKey - parameter index
//*           nParamAttr - parameter attribute index (See XS_ATTRIBUTE)
//*        pValue - ptr to the variable which receives the value
//*
//* Ret:   XS_SUCCESS - the parameter is successfully set.
//*        XS_E_NOT_SUPPORTED - the parameter is not supported on this camera cfg.
//*        XS_E_INVALID_CFG - invalid XS_SETTINGS structure.
//*
//*****************************************************************************
//
XS_ERROR XSTREAMAPI XsGetParameterAttribute(XS_HANDLE hCamera,
                                            PXS_SETTINGS pCfg,
                                            XS_PARAM nParamKey,
                                            XS_ATTRIBUTE nParamAttr,
                                            PXSULONG32 pValue);

//*****************************************************************************
//*
//*  XsGetParameter
//*
//* Descr: Gets a parameter from the configuration structure.
//*
//* Param: hCamera - handle to an open camera
//*        pCfg - pointer to a XS_SETTINGS structure
//*        nParamKey - parameter index
//*        pValue - ptr to the variable which receives the value
//*
//* Ret:   XS_SUCCESS - the parameter is successfully set.
//*        XS_E_NOT_SUPPORTED - the parameter is not supported on this camera cfg.
//*        XS_E_INVALID_CFG - invalid XS_SETTINGS structure.
//*
//*****************************************************************************
//
XS_ERROR XSTREAMAPI XsGetParameter(XS_HANDLE hCamera,
                                   PXS_SETTINGS pCfg,
                                   XS_PARAM nParamKey,
                                   PXSULONG32 pValue);

//*****************************************************************************
//*
//*  XsSetParameter
//*
//* Descr: sets a parameter into the configuration structure
//*
//* Param: hCamera - handle to an open camera
//*        pCfg - pointer to a XS_SETTINGS structure
//*        nParamKey - parameter index
//*        nValue - the value to write
//*
//* Ret:   XS_SUCCESS - the parameter is successfully set.
//*        XS_E_NOT_SUPPORTED - the parameter is not supported on this camera cfg.
//*           XS_E_READONLY - the parameter is read-only and cannot be changed
//*        XS_E_INVALID_VALUE - the parameter value is not valid
//*        XS_E_INVALID_CFG - invalid XS_SETTINGS structure.
//*
//*****************************************************************************
//
XS_ERROR XSTREAMAPI XsSetParameter(XS_HANDLE hCamera,
                                   PXS_SETTINGS pCfg,
                                   XS_PARAM nParamKey,
                                   XSULONG32 nValue);

//*****************************************************************************
//*
//*  XsReadUserDataFromFlash
//*
//* Descr: Read user data from the camera flash memory, if available
//*
//* Param: hCamera - handle to an open camera
//*        nType - destination ( 0=internal flash, 1=internal memory)
//*        nDataID - data unique ID (identifies user data) or start address (expressed in number of 2048 bytes blocks)
//*        nSize - size of data [bytes]
//*        pDataBuff - pointer to data buffer
//*
//* Ret:   XS_SUCCESS - configuration successfully validated.
//*        XS_E_NOT_SUPPORTED - not supported or not flash memory
//*        XS_E_INVALID_HANDLE - invalid camera handle
//*        XS_E_NOT_IN_FLASH - the user data is not stored in flash memory
//*        XS_E_HARDWARE_FAULT - error on driver
//*
//*****************************************************************************
//
XS_ERROR XSTREAMAPI XsReadUserDataFromFlash(XS_HANDLE hCamera,
                                            XSULONG32 nType,
                                            XSULONG32 nDataIDOrOffset,
                                            PXSULONG32 pnSize,
                                            void *pDataBuff);

//*****************************************************************************
//*
//*  XsWriteUserDataToFlash
//*
//* Descr: Writes user data to camera flash memory
//*
//* Param: hCamera - handle to an open camera
//*        nType - destination ( 0=internal flash, 1=internal memory)
//*        nDataID - data unique ID (identifies user data) or start address (expressed in number of 2048 bytes blocks)
//*        nSize - size of data [bytes]
//*        pDataBuff - pointer to data buffer
//*
//* Ret:   XS_SUCCESS - configuration successfully validated.
//*        XS_E_NOT_SUPPORTED - not supported or not flash memory
//*        XS_E_INVALID_HANDLE - invalid camera handle
//*        XS_E_HARDWARE_FAULT - error on driver
//*
//*****************************************************************************
//
XS_ERROR XSTREAMAPI XsWriteUserDataToFlash(XS_HANDLE hCamera,
                                           XSULONG32 nType,
                                           XSULONG32 nDataIDOrOffset,
                                           XSULONG32 nSize,
                                           void *pDataBuff);


//**************************************************************************************************
//* Preview Mode Grab Routines: grab and transfer to the PC memory like traditional cameras
//**************************************************************************************************

//*****************************************************************************
//*
//*  XsSynchGrab
//*
//* Descr: acquires one or more frames synchronously and returns when the frames
//*        have been acquired or an error occurs.
//*
//* Param: hCamera  - handle to an open camera
//*        pFrame - pointer to a valid XS_FRAME structure.
//*        nTimeOut - acquisition time out [ms]
//*
//* Ret:   XS_SUCCESS - frames successfully acquired
//*        XS_E_INVALID_HANDLE - invalid camera handle
//*        XS_E_BUFFER_TOO_SMALL - the buffer is too small for an acquisition
//*        XS_E_BUSY - the camera is busy
//*        XS_E_HARDWARE_FAULT: error using usb driver (SEVERE error)
//*
//*****************************************************************************
//
XS_ERROR XSTREAMAPI XsSynchGrab(XS_HANDLE hCamera,
                                PXS_FRAME pFrame,
                                XSULONG32 nTimeOut);

//*****************************************************************************
//*
//*  XsLive
//*
//* Descr: Enables/disables fast live mode. If fast live is on, images can be grabbed with
//*        XsMemoryPreview routine
//*
//* Param: hCamera  - handle to an open camera
//*        nCmd - start/stop live (see XS_LIVE)
//*
//* Ret:   XS_SUCCESS - command successfull
//*        XS_E_INVALID_HANDLE - invalid camera handle
//*        XS_E_NOT_SUPPORTED  - function is not supported
//*        XS_E_INVALID_ARGUMENTS - invalid function arguments
//*        XS_E_HARDWARE_FAULT - camera hardware fault
//*
//*****************************************************************************
//
XS_ERROR XSTREAMAPI XsLive(XS_HANDLE hCamera,
                           XS_LIVE nCmd);

//*****************************************************************************
//*
//*  XsQueueOneFrame
//*
//* Descr: Queues a frame (asynchronously). This function returns immediately.
//*        When the image buffer has been filled, a callback is called.
//*
//* Param: hCamera - handle to an open camera
//*        pFrame - pointer to a valid XS_FRAME structure.
//*        pfnCallback - callback routine pointer, it may be NULL
//*        nFlags - callback flags
//*        pUserData - parameter returned in callback, it may be a pointer
//*                    to user data.
//*
//* Ret:   XS_SUCCESS - frame successfully queued
//*        XS_E_INVALID_HANDLE - invalid camera handle
//*        XS_E_BUFFER_TOO_SMALL - the buffer is too small for an acquisition
//*        XS_E_QUEUE_FULL - the queue is full. nothing more can be added to it.
//*        XS_E_HARDWARE_FAULT: error using usb driver (SEVERE error)
//*
//*****************************************************************************
//
XS_ERROR XSTREAMAPI XsQueueOneFrame(XS_HANDLE hCamera,
                                    PXS_FRAME pFrame,
                                    XS_AsyncCallback pfnCallback,
                                    XSULONG32 nFlags,
                                    void *pUserData);

//*****************************************************************************
//*
//*  XsAbort
//*
//* Descr: any queued operation is aborted. The frame and settings queues are
//*        cleared.
//*
//* Param: hCamera - handle to an open camera
//*
//* Ret:   XS_SUCCESS - frame successfully queued
//*        XS_E_INVALID_HANDLE - invalid camera handle
//*
//*****************************************************************************
//
XS_ERROR XSTREAMAPI XsAbort(XS_HANDLE hCamera);


////////////////////////////////////////////////////////////////////////////////////////////////////
// Camera memory acquisition routines
////////////////////////////////////////////////////////////////////////////////////////////////////

//*****************************************************************************
//*
//*  XsMemoryStartGrab
//*
//* Descr: starts an acquisition in the camera memory. This function returns
//*        immediately. When the acquisition has been done,  a callback may
//*        be called (if passed as a parameter). if no callback address has been
//*        passed, the user may poll the camera status.
//*
//* Param: hCamera - handle to an open camera
//*        nStartAddLo - start rec address in camera memory - low order XSULONG32
//*        nStartAddHi - start rec address in camera memory - high order XSULONG32
//*        nFrames - number of frames to read
//*        nPreTrigFrames - number of frames to be acquired before the trigger
//*                         (valid in single trigger burst mode only)
//*        pfnCallback - callback routine pointer, it may be NULL
//*        nFlags - callback flags (see XS_CALLBACK_FLAGS)
//*        pUserData - parameter returned in callback, it may be a pointer
//*                    to user data.
//*
//* Ret:   XS_SUCCESS - the acquisition has started
//*        XS_E_INVALID_HANDLE - invalid camera handle
//*        XS_E_HARDWARE_FAULT: error using usb driver (SEVERE error)
//*
//*****************************************************************************
//
XS_ERROR XSTREAMAPI XsMemoryStartGrab(XS_HANDLE hCamera,
                                      XSULONG32 nStartAddLo,
                                      XSULONG32 nStartAddHi,
                                      XSULONG32 nFrames,
                                      XSULONG32 nPreTrigFrames,
                                      XS_AsyncCallback pfnCallback,
                                      XSULONG32 nFlags,
                                      void *pUserData);

//*****************************************************************************
//*
//*  XsMemoryStopGrab
//*
//* Descr: stops the current acquisition in the camera memory.
//*
//* Param: hCamera - handle to an open camera
//*        pnAcqFrames - pointer to the number of frames acquired so far
//*
//* Ret:   XS_SUCCESS - acquisition successfully stopped
//*        XS_E_INVALID_HANDLE - invalid camera handle
//*
//*****************************************************************************
//
XS_ERROR XSTREAMAPI XsMemoryStopGrab(XS_HANDLE hCamera,
                                     PXSULONG32 pnAcqFrames);

//*****************************************************************************
//*
//*  XsMemoryPreview
//*
//* Descr: previews the latest acquired frame during an acquisition
//*
//* Param: hCamera - handle to an open camera
//*        pFrame - pointer to a valid XS_FRAME structure (may be NULL).
//*        pnFrameIndex - pointer to the latest acquired frame index [0..n-1]
//*                       (may be NULL)
//*
//* Ret:   XS_SUCCESS - acquisition successfully stopped
//*        XS_E_INVALID_HANDLE - invalid camera handle
//*        XS_E_INVALID_ARGUMENTS - invalid arguments
//*        XS_E_BUSY - camera is busy
//*        XS_E_HARDWARE_FAULT - camera hardware fault
//*
//*****************************************************************************
//
XS_ERROR XSTREAMAPI XsMemoryPreview(XS_HANDLE hCamera,
                                    PXS_FRAME pFrame,
                                    PXSULONG32 pnFrameIndex);

//*****************************************************************************
//*
//*  XsMemoryReadFrame
//*
//* Descr: reads a frame from the camera memory.
//*
//* Param: hCamera - handle to an open camera
//*        nStartAddLo - starting address in camera memory - low order XSULONG32
//*        nStartAddHi - starting address in camera memory - high order XSULONG32
//*        nFrameIdxOrSize - index of the frame (all cams) or frame size (old N-cams)
//*        pDataBuff - pointer to data buffer
//*
//* Ret:   XS_SUCCESS - frames successfully read
//*        XS_E_INVALID_HANDLE - invalid camera handle
//*        XS_E_HARDWARE_FAULT - camera hardware fault
//*
//*****************************************************************************
//
XS_ERROR XSTREAMAPI XsMemoryReadFrame(XS_HANDLE hCamera,
                                      XSULONG32 nStartAddLo,
                                      XSULONG32 nStartAddHi,
                                      XSULONG32 nFrameIdxOrSize,
                                      void* pDataBuff);

//*****************************************************************************
//*
//*  XsMemoryReadFromDisk
//*
//* Descr: reads a frame from the camera memory.
//*
//* Param: hCamera - handle to an open camera
//*        nMemDstAddLo, nMemDstAddHi - destination address in camera memory
//*        nDiskSrcAddLo, nDiskSrcAddHi - source address in camera disk
//*        nStartIdx - index of the first image to transfer
//*        nStopIdx - index of the last image to transfer
//*        pfnCallback - callback routine pointer, it may be NULL
//*        pUserData - parameter returned in callback, it may be a pointer
//*                    to user data.
//*
//* Ret:   XS_SUCCESS - frames successfully read
//*        XS_E_INVALID_HANDLE - invalid camera handle
//*        XS_E_NOT_SUPPORTED - the routine is not supported
//*        XS_E_HARDWARE_FAULT - camera hardware fault
//*
//*****************************************************************************
//
XS_ERROR XSTREAMAPI XsMemoryReadFromDisk(XS_HANDLE hCamera,
                                         XSULONG32 nMemDstAddLo,
                                         XSULONG32 nMemDstAddHi,
                                         XSULONG32 nDiskSrcAddLo,
                                         XSULONG32 nDiskSrcAddHi,
                                         XSULONG32 nStartIdx,
                                         XSULONG32 nStopIdx,
                                         XS_ProgressCallback pfnCallback,
                                         void *pUserData);

//*****************************************************************************
//*
//*  XsMemoryDownloadRawFrame
//*
//* Descr: downloads one image frame from the camera memory to a RAW file.
//*
//* Param: hCamera - handle to an open camera
//*        szRawFilePath - full path of the raw file (without extension)
//*        nStartAddLo - starting address in camera memory - low order XSULONG32
//*        nStartAddHi - starting address in camera memory - high order XSULONG32
//*        nFrameIdx - index of the frame in camera memory (position)
//*        nPageIdx - index of the image in the file (0 to N-1)
//*        nTotFrames - total number of frames to download
//*
//* Ret:   XS_SUCCESS - frames successfully read
//*        XS_E_INVALID_HANDLE - invalid camera handle
//*        XS_E_BUSY - the camera is busy and cannot donwload
//*        XS_E_HARDWARE_FAULT - camera hardware fault
//*
//*****************************************************************************
//
XS_ERROR XSTREAMAPI XsMemoryDownloadRawFrame(XS_HANDLE hCamera,
                                             const char *lpszRawFilePath,
                                             XSULONG32 nStartAddLo,
                                             XSULONG32 nStartAddHi,
                                             XSULONG32 nFrameIdx,
                                             XSULONG32 nPageIdx,
                                             XSULONG32 nTotFrames);


//*****************************************************************************
//*
//*  XsMemoryReadTriggerPosition
//*
//* Descr: Reads position of the trigger frame in the camera memory.
//*
//* Param: hCamera - handle to an open camera
//*        pnTriggerPosLo - pointer to the trigger position address - low order XSULONG32
//*        pnTriggerPosHi - pointer to the trigger position address - high order XSULONG32
//*        pnTriggerIndex - pointer to the trigger position index
//*        pnTriggerTime  - pointer to the trigger position time relative to the 
//*                         sync signal [ns]
//*
//* Ret:   XS_SUCCESS - frames successfully read
//*        XS_E_INVALID_HANDLE - invalid camera handle
//*        XS_E_INVALID_ARGUMENTS - invalid arguments
//*        XS_E_HARDWARE_FAULT - the status cannot be read
//*
//*****************************************************************************
//
XS_ERROR XSTREAMAPI XsMemoryReadTriggerPosition(XS_HANDLE hCamera,
                                                PXSULONG32 pnTriggerPosLo,
                                                PXSULONG32 pnTriggerPosHi,
                                                PXSULONG32 pnTriggerIndex,
                                                PXSULONG32 pnTriggerTime);

//*****************************************************************************
//*
//*  XsEraseMemory
//*
//* Descr: Erases the camera memory.
//*
//* Param: hCamera - handle to an open camera
//*
//* Ret:   XS_SUCCESS - configuration successfully validated.
//*        XS_E_INVALID_HANDLE - invalid camera handle
//*        XS_E_HARDWARE_FAULT - error on driver
//*
//*****************************************************************************
//
XS_ERROR XSTREAMAPI XsEraseMemory(XS_HANDLE hCamera);

//*****************************************************************************
//*
//*  XsEraseDisk
//*
//* Descr: Erases the camera disk.
//*
//* Param: hCamera - handle to an open camera
//*
//* Ret:   XS_SUCCESS - configuration successfully validated.
//*        XS_E_INVALID_HANDLE - invalid camera handle
//*        XS_E_NOT_SUPPORTED - the routine is not supported
//*        XS_E_HARDWARE_FAULT - error on driver
//*
//*****************************************************************************
//
XS_ERROR XSTREAMAPI XsEraseDisk(XS_HANDLE hCamera);

//*****************************************************************************
//*
//*  XsTrigger
//*
//* Descr: Issues a software trigger to the camera.
//*
//* Param: hCamera - handle to an open camera
//*
//* Ret:   XS_SUCCESS - frames successfully acquired
//*        XS_E_INVALID_HANDLE - invalid camera handle
//*        XS_E_NOT_SUPPORTED - the API si not supported
//*        XS_E_NOT_RECORDING - the camera is not in record mode
//*        XS_E_BUSY - the camera is busy
//*        XS_E_HARDWARE_FAULT: error using usb driver (SEVERE error)
//*
//*****************************************************************************
//
XS_ERROR XSTREAMAPI XsTrigger(XS_HANDLE hCamera);

//*****************************************************************************
//*
//*  XsGetHardwareError
//*
//* Descr: returns last encountered vendor specific hardware error.
//*
//* Param: hCamera - handle to an open camera
//*        pszBuffer - address of a buffer where to store description string
//*           nSize - the size of provided buffer
//*
//* Ret:   error code
//*
//*****************************************************************************
//
XSULONG32 XSTREAMAPI XsGetHardwareError(XS_HANDLE hCamera,
                                        char* pszBuffer,
                                        XSULONG32 nSize);


//*****************************************************************************
//*
//*  XsCalibrateNoiseReduction
//*
//* Descr: calibrates the camera, reduces the fixed noise pattern and
//*        reduces differences due to pixel sensitivity
//*
//* Param: hCamera - handle to an open camera
//*        nOpCode - operation code (see XS_CALIB_OPCODE)
//*        pfnCallback - callback routine pointer, it may be NULL
//*        pUserData - parameter returned in callback, it may be a pointer
//*                    to user data.
//*
//* Ret:   XS_SUCCESS - frames successfully read
//*        XS_E_INVALID_HANDLE - invalid camera handle
//*        XS_E_NOT_SUPPORTED - the routine is not supported
//*        XS_E_HARDWARE_FAULT - the status cannot be read
//*        XS_E_NOT_IN_FLASH - if the opcode is XS_C_FILE_DOWNLOAD, the file
//*                            is not stored in flash memory.
//*        XS_E_ABORTED - the procedure has been aborted.
//*
//*****************************************************************************
//
XS_ERROR XSTREAMAPI XsCalibrateNoiseReduction(XS_HANDLE hCamera,
                                              XSULONG32 nOpCode,
                                              XS_ProgressCallback pfnCallback,
                                              void *pUserData);

//*****************************************************************************
//*
//*  XsReadGPSTiming
//*
//* Descr: Reads GPS/IRIG timing info from the current frame or from the camera
//*
//* Param: hCamera - handle to an open camera
//*        nOption - 0: read from current frame, 1: read from camera
//*        pGPS - pointer to the XS_GPSTIMING structure
//*
//* Ret:   XS_SUCCESS - timing successfully read
//*        XS_E_INVALID_HANDLE - invalid camera handle
//*        XS_E_INVALID_VALUE - invalid parameter
//*        XS_E_NOT_SUPPORTED - the routine is not supported
//*
//*****************************************************************************
//
XS_ERROR XSTREAMAPI XsReadGPSTiming(XS_HANDLE hCamera,
                                    XSULONG32 nOption,
                                    PXS_GPSTIMING pGPS);

//*****************************************************************************
//*
//*  XsReset
//*
//* Descr: resets the camera
//*
//* Param: hCamera - handle to an open camera
//*
//* Ret:   XS_SUCCESS - frames successfully read
//*        XS_E_INVALID_HANDLE - invalid camera handle
//*        XS_E_HARDWARE_FAULT - the status cannot be read
//*
//*****************************************************************************
//
XS_ERROR XSTREAMAPI XsReset(XS_HANDLE hCamera);

//*****************************************************************************
//*
//*  XsEnableDiagnosticTrace
//*
//* Descr: enables/disables the diagnostic trace
//*
//* Param: hCamera - handle to an open camera
//*        pszTraceFilePath - full path of the trace file (text)
//*           nEnable - enable (1) disable (0) flag
//*
//* Ret:   XS_SUCCESS - frames successfully acquired
//*        XS_E_INVALID_HANDLE - invalid camera handle
//*
//*****************************************************************************
//
XS_ERROR  XSTREAMAPI XsEnableDiagnosticTrace(XS_HANDLE hCamera,
                                             char* pszTraceFilePath,
                                             XSULONG32 nEnable);

//*****************************************************************************
//*
//*  XsEnableRawMode
//*
//* Descr: enables/disables the raw data mode
//*
//* Param: hCamera - handle to an open camera
//*           nEnable - enable (1) disable (0) flag
//*
//* Ret:   XS_SUCCESS - frames successfully acquired
//*        XS_E_INVALID_HANDLE - invalid camera handle
//*
//*****************************************************************************
//
XS_ERROR  XSTREAMAPI XsEnableRawMode(XS_HANDLE hCamera,
                                     XSULONG32 nEnable);

//*****************************************************************************
//*
//*  XsGetCameraStatus
//*
//* Descr: Reads the camera status
//*
//* Param: hCamera - handle to an open camera
//*        pnIsBusy - pointer to the busy variable (0 or 1)
//*        pnStatus - pointer to the status variable
//*        pnErrCode - pointer to error code variable (can be NULL)
//*        pnInfo1,2,3 - pointers to information variables (can be NULL)
//*
//* Ret:   XS_SUCCESS - configuration successfully validated.
//*        XS_E_INVALID_HANDLE - invalid camera handle
//*        XS_E_INVALID_ARGUMENTS - invalid arguments
//*        XS_E_HARDWARE_FAULT - error on driver
//*
//*****************************************************************************
//
XS_ERROR XSTREAMAPI XsGetCameraStatus(XS_HANDLE hCamera,
                                      PXSULONG32 pnIsBusy,
                                      PXSULONG32 pnStatus,
                                      PXSULONG32 pnErrCode,
                                      PXSULONG32 pnInfo1,
                                      PXSULONG32 pnInfo2,
                                      PXSULONG32 pnInfo3);

//*****************************************************************************
//*
//*  XsSetAnnouncementCallback
//*
//* Descr: Sets the announcement callback. See the anouncement string list above
//*
//* Param: hCamera - handle to an open camera
//*        pfnCallback - callback routine pointer, it may be NULL
//*        pUserData - parameter returned in callback, it may be a pointer
//*                    to user data.
//*
//* Ret:   XS_SUCCESS - configuration successfully validated.
//*        XS_E_INVALID_HANDLE - invalid camera handle
//*        XS_E_HARDWARE_FAULT - error on driver
//*
//*****************************************************************************
//
XS_ERROR XSTREAMAPI XsSetAnnouncementCallback(XS_HANDLE hCamera,
                                              XS_AnnouncementCallback pfnCallback,
                                              void *pUserData);

//*****************************************************************************
//*
//*  XsReadBorderData
//*
//* Descr: Reads the border data (HG-only)
//*
//* Param: hCamera - handle to an open camera
//*        pDataBuff - pointer to the border data buffer
//*        nSize - size of the border data buffer
//*
//* Ret:   XS_SUCCESS - configuration successfully validated.
//*        XS_E_INVALID_HANDLE - invalid camera handle
//*        XS_E_HARDWARE_FAULT - error on driver
//*
//*****************************************************************************
//
XS_ERROR XSTREAMAPI XsReadBorderData(XS_HANDLE hCamera,
                                     void* pDataBuff,
                                     XSULONG32 nSize);

//*****************************************************************************
//*
//*  XsAttach
//*
//* Descr: Attach to camera. Handle simultaneous connection from different
//*        computers. For HG,Y,N,NR,NX cameras
//*
//* Param: hCamera - handle to an open camera
//*
//* Ret:   XS_SUCCESS - frames successfully read
//*        XS_E_INVALID_HANDLE - invalid camera handle
//*        XS_E_HARDWARE_FAULT - the status cannot be read
//*
//*****************************************************************************
//
XS_ERROR XSTREAMAPI XsAttach(XS_HANDLE hCamera);

//*****************************************************************************
//*
//*  XsGetAddressList
//*
//* Descr: Reads a list of image addresses. Supported by old N3 and N4 cameras.
//*
//* Param: hCamera - handle to an open camera
//*        nStartIdx - starting frame index
//*        nAddressCount - number of addresses to read
//*        pnAddrList - pointer to buffer where to store addresses list
//*
//* Ret:   XS_SUCCESS - frames successfully read
//*        XS_E_INVALID_HANDLE - invalid camera handle
//*        XS_E_HARDWARE_FAULT - camera hardware fault
//*
//*****************************************************************************
//
XS_ERROR XSTREAMAPI XsGetAddressList(XS_HANDLE hCamera,
                                     XSULONG32 nStartIdx,
                                     XSULONG32 nAddressCount,
                                     PXSUINT64 pnAddrList);

//*****************************************************************************
//*
//*  XsConfigureWriteToDisk
//*
//* Descr: configures the "Write to Disk" parameters.
//*
//* Param: hCamera - handle to an open camera
//*        nEnable - on/off flag
//*        pCfg - pointer to XS_W2DCFG structure
//*        pfnCallback - pointer to XS_StreamingCallback
//*        pUserData - pointer to user data (returned in callback)
//*
//* Ret:   XS_SUCCESS - frames successfully read
//*        XS_E_INVALID_HANDLE - invalid camera handle
//*        XS_E_NOT_SUPPORTED - the option is not supported
//*
//*****************************************************************************
//
XS_ERROR XSTREAMAPI XsConfigureWriteToDisk(XS_HANDLE hCamera,
                                           XSULONG32 nEnable,
                                           PXS_W2DCFG pCfg,
                                           XS_StreamingCallback pfnCallback,
                                           void *pUserData);

//*****************************************************************************
//*
//*  XsReadToVideo
//*
//* Descr: sends a command to the HDMI output to read a frame and display it.
//*        Supported byt Y cameras only.
//*
//* Param: hCamera - handle to an open camera
//*        nStartAddLo - starting address in camera memory - low order XSULONG32
//*        nStartAddHi - starting address in camera memory - high order XSULONG32
//*        nFrameIdx - index of the frame (if 0xFFFFFFF it snaps an image)
//*
//* Ret:   XS_SUCCESS - frames successfully read
//*        XS_E_INVALID_HANDLE - invalid camera handle
//*        XS_E_NOT_SUPPORTED - the option is not supported
//*        XS_E_BUSY - the operation cannot be completed because the camera is busy
//*        XS_E_HARDWARE_FAULT - camera hardware fault
//*
//*****************************************************************************
//
XS_ERROR XSTREAMAPI XsReadToVideo(XS_HANDLE hCamera,
                                  XSULONG32 nStartAddLo,
                                  XSULONG32 nStartAddHi,
                                  XSULONG32 nFrameIdx);


//*****************************************************************************
//*
//*  XsLoadLookupTable
//*
//* Descr: Loads a custom Lookup Table.
//*
//* Param: hCamera - handle to an open camera
//*        pnTable - pointer to the table (array of ushort)
//*        nSize - size of the table
//*
//* Ret:   XS_SUCCESS - frames successfully read
//*        XS_E_INVALID_HANDLE - invalid camera handle
//*        XS_E_INVALID_ARGUMENTS - invalid arguments
//*
//*****************************************************************************
//
XS_ERROR XSTREAMAPI XsLoadLookupTable(XS_HANDLE hCamera,
                                      unsigned short *pnTable,
                                      XSULONG32 nSize);

//*****************************************************************************
//*
//*  XsGetBrocParameters
//*
//* Descr: reads addresses and indexes of current BROC sections
//*        For cameras that support hardware BROC (Y,NR,NX).
//*
//* Param: hCamera  - handle to an open camera
//*           pBrocSectArray - array of XS_BROC_SECTION items
//*        nSize - number of items in the array
//*
//* Ret:   XS_SUCCESS - resizer successfully activated
//*        XS_E_INVALID_HANDLE - invalid camera handle
//*        XS_E_INVALID_ARGUMENTS - invalid arguments
//*        XS_E_NOT_SUPPORTED - the routine is not supported
//*
//*****************************************************************************
//
XS_ERROR XSTREAMAPI XsGetBrocParameters(XS_HANDLE hCamera,
                                        PXS_BROC_SECTION pBrocSectArray,
                                        XSULONG32 nSize);

//*****************************************************************************
//*
//*  XsVideoPlayback
//*
//* Descr: starts or stops fast playback on the HDMI output
//*
//* Param: hCamera  - handle to an open camera
//*        nOption - 0:stop,  1:forward, 2:rewind (see XS_VIDEO_PB)
//*        nStartAddLo - starting address in camera memory - low order XSULONG32
//*        nStartAddHi - starting address in camera memory - high order XSULONG32
//*        nFrames - number of frames to play back
//*        nStartFrameIdx - index of first frame.
//*        nStopFrameIdx - index of last frame.
//*
//* Ret:   XS_SUCCESS - resizer successfully activated
//*        XS_E_INVALID_HANDLE - invalid camera handle
//*        XS_E_INVALID_ARGUMENTS - invalid arguments
//*        XS_E_NOT_SUPPORTED - the routine is not supported
//*        XS_E_HARDWARE_FAULT - the command returns error
//*
//*****************************************************************************
//
XS_ERROR XSTREAMAPI XsVideoPlayback(XS_HANDLE hCamera,
                                    XSULONG32 nOption,
                                    XSULONG32 nStartAddLo,
                                    XSULONG32 nStartAddHi,
                                    XSULONG32 nFrames,
                                    XSULONG32 nStartFrameIdx,
                                    XSULONG32 nStopFrameIdx);

#ifdef __cplusplus
} // end extern "C"
#endif


#endif    // __XSTREAMVISION_H__
